#include <stdio.h>
#include <string.h>
#include "def.h"
#include "option.h"
#include "2443addr.h"
#include "system.h"
#include "pll.h"
#include "console.h"
#include "hs_mmc.h"

#define POL_Ver1		0
#define INT_Ver1		1
#define DMA_Ver1	2
#define SDI_Tx_buffer_HSMMC (_NONCACHE_STARTADDRESS)
#define SDI_Rx_buffer_HSMMC (_NONCACHE_STARTADDRESS+(0x300000))
#define SDI_Compare_buffer_HSMMC (_NONCACHE_STARTADDRESS+(0x600000))


#define Card_OneBlockSize_ver1 512

// Global variables
unsigned int *Tx_buffer_HSMMC;	
unsigned int *Rx_buffer_HSMMC;
unsigned int *Compare_buffer_HSMMC;

volatile unsigned int rd_cnt_HSMMC;
volatile unsigned int wt_cnt_HSMMC;
volatile unsigned int BlockNum_HSMMC=0;

volatile unsigned int WriteBlockCnt_INT=0;
volatile unsigned int ReadBlockCnt_INT=0;
volatile unsigned int WRITEINT_DONE=0;
volatile unsigned int READINT_DONE=0;
volatile unsigned int COMPARE_INT_DONE=0;
volatile unsigned int CompareCnt_INT=0;
volatile unsigned int BufferBoundary_INT_Cnt=0;

volatile unsigned int HS_DMA_END = 0;
volatile unsigned int HS_CARD_DETECT = 0;

volatile int OCRcheck=0;
volatile int ThisIsMmc=0;
volatile int m_uRca=0;
volatile int m_ucMMCSpecVer = 0;
volatile int SDSpecVer;

extern	unsigned int ARMCLK, HCLK, PCLK;

//////////////////////////////////////////////////////////////////////////

#define NEW_SD_CARD		1
#define BUFFER_BOUNDARY	0
#define HCLK_OPERATION		0

//////////////////////////////////////////////////////////////////////////

void * func_HSMMC_test[][2]=
{	
    (void *)HS_MMC_WriteTest,       	    "SDIO Write          ",
    (void *)HS_MMC_ReadTest,           	"SDIO Read           ",
    (void *)HS_MMC_EraseBlock,            "SDIO Block Erase    ",
    0,0
};

void Test_HS_MMC(void)
{  
	int i;

	HS_MMC_SETGPIO();	

	HS_MMC_CardDetect();	

	HS_MMC_Reset();	
	
	if(!HS_MMC_init())
	{
		printf("\nCard initial fail\n");
	}

	while(1)
	{
		i=0;
		printf("\n\n");
		while(1)
		{   //display menu
	   		printf("%2d:%s",i,func_HSMMC_test[i][1]);
	    	i++;
	    	if((int)(func_HSMMC_test[i][0])==0)
	    	{
				printf("\n");
				break;
	    	}
	    	if((i%3)==0)
	    	printf("\n");
		}

		printf("\nSelect (\"-1\" to exit) : ");
		i = GetIntNum();
		if(i==-1) 
	   		break;		// return.
		if( (i<((sizeof(func_HSMMC_test)-1)/8)) )	// select and execute...
	    	( (void (*)(void)) (func_HSMMC_test[i][0]) )();
	}
	
}


int HS_MMC_init(void)
{
	int i, Card_State, blocksize,BD_BUS_WIDTH,ucBusWidth;
	char key;

	ClockOnOff(0);// SDCLK Stop
	//rSCLKCON |=(1<<13); error Mango  extclk no have
	rSCLKCON |=(1<<12); 
     
	//SetClock(SD_HCLK, 0x80);
	SetClock(SD_HCLK, 0x20);
	
	rHM_TIMEOUTCON = 0xe;

	HostCtrlSpeedMode(NORMAL);

	InterruptEnable(0xff, 0xff);

	printf("rHM_NORINTSTS = %x\n",rHM_NORINTSTS);
	
	IssueCommand(0,0x00,0);//Idle State
	printf("\nEnter to the Idle State\n");

	OCRcheck =1;

	if(SetMMCOCR())
	{
		ThisIsMmc=1;
		printf("MMC card is detected\n");
	}
	else if(SetSDOCR())
	{
		ThisIsMmc=0;
		printf("SD card is detected\n");
	}
	else
		return 0;

	OCRcheck =0;
	
	// Check the attached card and place the card in the IDENT state rHM_RSPREG0
	IssueCommand(2,0,0);	
	printf("\n - Product Name : %c%c%c%c%c%c\n",((rHM_RSPREG2>>24)&0xFF),((rHM_RSPREG2>>16)&0xFF),((rHM_RSPREG2>>8)&0xFF),(rHM_RSPREG2&0xFF),((rHM_RSPREG1>>24)&0xFF),((rHM_RSPREG1>>16)&0xFF));		

	// Send RCA(Relative Card Address). It places the card in the STBY state
	m_uRca = (ThisIsMmc) ? 0x0001 : 0x0000;
 	IssueCommand(3,m_uRca,0);	
	
	if(!ThisIsMmc)
 		{
			m_uRca = (rHM_RSPREG0>>16)&0xFFFF;			
			printf("=>  RCA=0x%x\n", m_uRca);			
 		}	
	
	printf("\nEnter to the Stand-by State\n");
	
	IssueCommand(9, m_uRca, 0);//Send CSD
	
	DisplayCardInformation();

	IssueCommand(7, m_uRca, 0);	
	//printf("\nEnter to the Transfer State\n");		
		
//////////////////////// Operating Clock setting ////////////////////////////
#if HCLK_OPERATION
	ClockConfig(SD_HCLK, 1);// Divisor 1 = Base clk /2	,Divisor 2 = Base clk /4, Divisor 4 = Base clk /8 ...		
#else
	rLOCKCON1=0x800; 
	rCLKSRC|=(1<<6);  // EPLL Output
	//SetEPLL(42, 1, 1);//100MHz
	SetEPLL(40, 1, 1);//96MHz
	rEPLLCON &= ~(1<<24);  // EPLL ON 
	rCLKDIV1 = (rCLKDIV1 & ~(0x3<<6)) | (0x0<<6);
	rMISCCR = rMISCCR & ~(0x7<<8) | (1<<8);
	rGPHCON = rGPHCON & ~(0x3<<28) | (1<<29);
	ClockConfig(SD_EPLL, 1);// Divisor 1 = Base clk /2 ,Divisor 2 = Base clk /4, Divisor 4 = Base clk /8 ...	
	//ClockConfig(SD_EPLL, 2);// Divisor 1 = Base clk /2	,Divisor 2 = Base clk /4, Divisor 4 = Base clk /8 ...

/*
	rSCLKCON |=(1<<13);
	SetClock(SD_EXTCLK, 0);// HCLK = 100MHz, SD CLK = 390KHz
	ClockOnOff(0); // when change the sd clock frequency, need to stop sd clock.
*/
#endif

///////////////////////////////////////////////////////////////////////
	while (!SetDataTransferWidth());	

	while (!IsCardInProgrammingState());

	printf("\nSet a One Block size\n");
	while(!IssueCommand(16, 512, 0));//Set the One Block size	

	rHM_NORINTSTS = 0xffff;
}


void HS_MMC_WriteTest(void)
{
	U32 i, j, StartAddr,  OneBlockSize, Offset, Testmode;
	U32 TotalWriteByte, WriteBlockCnt =0;
	U32 transtime;

	U32 uTxBufAddr = SDI_Tx_buffer_HSMMC;
	U32 uCompareBufAddr = SDI_Compare_buffer_HSMMC;
	
	wt_cnt_HSMMC=0;
	BlockNum_HSMMC = 0;
	wt_cnt_HSMMC = 0;
	WriteBlockCnt_INT = 0;
	HS_DMA_END = 0;
	
	printf("\nSD/MMC block write test\n");

	printf("\n0:Polling write   1:Interrupt write   2:DMA write");
	printf("\nSelect the test mode : ");
    
       Testmode=GetIntNum();
    	
	printf("\nInput Write Start block number : ");
	StartAddr = GetIntNum();

	printf("Input Write Data Offset : ");
	Offset = GetIntNum();

	while((BlockNum_HSMMC == 0) || (BlockNum_HSMMC > 65535))
	{
		printf("Input number of Block [1~65535] : ");
		BlockNum_HSMMC = GetIntNum();
	}

	OneBlockSize = Card_OneBlockSize_ver1;
	
	Tx_buffer_HSMMC = (U32 *)SDI_Tx_buffer_HSMMC;
    	for(i=0 ; i<(OneBlockSize*BlockNum_HSMMC)/4 ; i++)
    		{
    		*(Tx_buffer_HSMMC+i) = (i+Offset);   	
   		}				
	
	switch(Testmode)
		{
			case  POL_Ver1:
				printf("\nPolling mode data write\n");

				SetBlockSizeReg(7, 512); // Maximum DMA Buffer Size, Block Size
				SetBlockCountReg(BlockNum_HSMMC); // Block Numbers to Write
				SetArgumentReg(StartAddr*512); // Card Address to Write

				if(BlockNum_HSMMC == 1)//single block
					{
						SetTransferModeReg(0, 0, 1, 1, 0);
						SetCommandReg(24, 0); 	
					}
				else//multi block
					{
						SetTransferModeReg(1, 0, 1, 1, 0);
						SetCommandReg(25, 0); 
					}

				if (!WaitForCommandComplete())
					{
						printf("\nCommand is NOT completed\n");
					}
				ClearCommandCompleteStatus();

				for(j=0; j<BlockNum_HSMMC; j++)
					{
						if (!WaitForBufferWriteReady())
							printf("WriteBuffer NOT Ready\n");
						else
							ClearBufferWriteReadyStatus();

						for(i=0; i<512/4; i++)//512 byte should be writed.
							{
								rHM_BDATA = *Tx_buffer_HSMMC++;	
				  		 		wt_cnt_HSMMC++;						
							}
						WriteBlockCnt ++;
						//printf("\nWrite block count = %d", WriteBlockCnt);
					}

				TotalWriteByte = wt_cnt_HSMMC *4;
				printf("\nWrite count=%dByte\n",TotalWriteByte);
				if(!WaitForTransferComplete())
					{
					printf(("Transfer is NOT Complete\n"));
					}
				ClearTransferCompleteStatus();
				
				break;

			case INT_Ver1:

				printf("\n Interrupt mode data write\n");
				pISR_SDI_1=(unsigned)HS_WRITE_INT;
				
				SetBlockSizeReg(7, 512); // Maximum DMA Buffer Size, Block Size
				SetBlockCountReg(BlockNum_HSMMC); // Block Numbers to Write
				SetArgumentReg(StartAddr*512); // Card Address to Write

				StartStopwatch();
				
				if(BlockNum_HSMMC == 1)//single block
					{
						SetTransferModeReg(0, 0, 1, 1, 0);
						SetCommandReg(24, 0); 	
					}
				else//multi block
					{
						SetTransferModeReg(1, 0, 1, 1, 0);
						SetCommandReg(25, 0); 
					}

				if (!WaitForCommandComplete())
					{
						printf("\nCommand is NOT completed\n");
					}
				ClearCommandCompleteStatus();

				printf("\nInterupt mode Write Start\n");
				rINTMSK &= ~(BIT_SDI1);
				rHM_NORINTSIGEN = rHM_NORINTSIGEN & ~(0xffff) | BUFFER_WRITEREADY_SIG_INT_EN;	

				while(!WRITEINT_DONE);

				//printf("\nInterupt mode Write End\n");
				
				if(!WaitForTransferComplete())
					{
					printf(("Transfer is NOT Complete\n"));
					}
				ClearTransferCompleteStatus();
				

				transtime = EndStopwatch();
				CalculationBPS_HSMMC(transtime);
				
	    			break;
					
			case DMA_Ver1:

				#if BUFFER_BOUNDARY
				
				pISR_SDI_1 = (unsigned)HS_DMA_INT;
				rINTMSK &= ~(BIT_SDI1);				
				rHM_NORINTSTSEN |= (DMA_STS_INT_EN |TRANSFERCOMPLETE_STS_INT_EN);
				rHM_NORINTSIGEN = rHM_NORINTSIGEN & ~(0xffff) | DMA_SIG_INT_EN |TRANSFERCOMPLETE_SIG_INT_EN;
				
				SetSystemAddressReg(SDI_Tx_buffer_HSMMC);// AHB System Address For Write
				SetBlockSizeReg(0, 512); // 4K Byte Buffer Boundary, Block Size				
				SetBlockCountReg(BlockNum_HSMMC); // Block Numbers to Write	
				SetArgumentReg(StartAddr*512);// Card Start Block Address to Write				

				#else		

				pISR_SDI_1 = (unsigned)HS_DMA_INT;
				
				rINTMSK &= ~(BIT_SDI1);				
				//rHM_NORINTSTSEN &= ~(DMA_STS_INT_EN|BLOCKGAP_EVENT_STS_INT_EN);
				rHM_NORINTSTSEN = (rHM_NORINTSTSEN & ~(0xffff)) |
									BUFFER_READREADY_STS_INT_EN | 
									BUFFER_WRITEREADY_STS_INT_EN |
									TRANSFERCOMPLETE_STS_INT_EN |
									COMMANDCOMPLETE_STS_INT_EN;
				rHM_NORINTSIGEN = (rHM_NORINTSIGEN & ~(0xffff)) | TRANSFERCOMPLETE_SIG_INT_EN;				
			
				SetSystemAddressReg(SDI_Tx_buffer_HSMMC);// AHB System Address For Write
				SetBlockSizeReg(7, 512); // Maximum DMA Buffer Size, Block Size				
				SetBlockCountReg(BlockNum_HSMMC); // Block Numbers to Write	
				SetArgumentReg(StartAddr*512);// Card Start Block Address to Write				
				
				#endif			

				//StartStopwatch();

				if (BlockNum_HSMMC == 1)
				{
					SetTransferModeReg(0, 0, 1, 1, 1);
					SetCommandReg(24, 0); // CMD25: Single-Write
				}
				else
				{	// (u32 MultiBlk, u32 DataDirection, u32 AutoCmd12En, u32 BlockCntEn, u32 DmaEn)
					SetTransferModeReg(1, 0, 1, 1, 1);
					SetCommandReg(25, 0); // CMD25: Multi-Write
				}
				
				if (!WaitForCommandComplete())
				{
					printf("\nCommand is NOT completed\n");
				}
				ClearCommandCompleteStatus();		
				
				while(!HS_DMA_END);

				if(!WaitForTransferComplete())
					{
					printf(("Transfer is NOT Complete\n"));
					}
				ClearTransferCompleteStatus();
				
				rHM_NORINTSTS |= (1<<3);
				
				//transtime = EndStopwatch();
				//CalculationBPS_HSMMC(transtime);
				
				printf(("\nDMA Write End\n"));
				
				break;	
				
			default :
				break;					
		}	

	DataRead_ForCompare(StartAddr);
	DataCompare_HSMMC(uTxBufAddr, uCompareBufAddr, BlockNum_HSMMC * 128);
	
	BlockNum_HSMMC = 0;
	wt_cnt_HSMMC = 0;
	WriteBlockCnt_INT = 0;
	HS_DMA_END = 0;
	BufferBoundary_INT_Cnt = 0;

	CompareCnt_INT = 0;
	Compare_buffer_HSMMC = 0;
}


void HS_MMC_ReadTest(void)
{
	U32 i, j, StartAddr,  OneBlockSize, Offset, Testmode, Addr_temp;
	U32 TotalReadByte, WriteBlockCnt=0;
	U32 status;

	rd_cnt_HSMMC=0;	
	HS_DMA_END = 0;	
	BlockNum_HSMMC = 0;
	rd_cnt_HSMMC = 0;
	ReadBlockCnt_INT = 0;	

	printf("\nSD/MMC block Read test\n");

	printf("\n0:Polling Read   1:Interrupt Read   2:DMA Read");
	printf("\nSelect the test mode : ");
    
       Testmode=GetIntNum();
    	
	printf("\nInput Write Start block number : ");
	StartAddr = GetIntNum();
	Addr_temp = StartAddr;	

	while((BlockNum_HSMMC == 0) || (BlockNum_HSMMC > 65535))
	{
		printf("Input number of block[1~4095] : ");
		BlockNum_HSMMC = GetIntNum();
	}
	
	OneBlockSize = Card_OneBlockSize_ver1;

	Rx_buffer_HSMMC = (U32 *)SDI_Rx_buffer_HSMMC;
    	for(i=0 ; i<(OneBlockSize*BlockNum_HSMMC)/4 ; i++)	
		*(Rx_buffer_HSMMC+i) = 0x0;   	

	switch(Testmode)
		{
			case  POL_Ver1:
				printf("\nPolling mode data read\n");

				while (!IsCardInProgrammingState());

				SetBlockSizeReg(7, 512); // Maximum DMA Buffer Size, Block Size
				SetBlockCountReg(BlockNum_HSMMC); // Block Numbers to Write
				SetArgumentReg(StartAddr*512); // Card Address to Write

				if(BlockNum_HSMMC == 1)//single block
					{
						printf("Single block read\n");
						SetTransferModeReg(0, 1, 0, 1, 0);
						SetCommandReg(17, 0); // CMD17: Single-Read						
					}
				else//multi block
					{
						printf("Multi block read\n");
						SetTransferModeReg(1, 1, 1, 1, 0);
						SetCommandReg(18, 0); // CMD18: Multi-Read
					}

				while (!WaitForCommandComplete());
				ClearCommandCompleteStatus();
				
				for(j=0; j<BlockNum_HSMMC; j++)
					{
						if (!WaitForBufferReadReady())
							printf("ReadBuffer NOT Ready\n");
						else
							ClearBufferReadReadyStatus();
						for(i=0; i<512/4; i++)
						{
							*Rx_buffer_HSMMC++ = rHM_BDATA;
							rd_cnt_HSMMC++;
						}
						WriteBlockCnt ++;
						printf("\nWrite block count = %d", WriteBlockCnt);
					}

				TotalReadByte = wt_cnt_HSMMC *4;
				printf("\nWrite count=%dByte\n",TotalReadByte);
				
				printf("\nWrite count=%x\n",rd_cnt_HSMMC);
				if(!WaitForTransferComplete())
					{
					printf(("Transfer NOT Complete\n"));
					}
				ClearTransferCompleteStatus();

				break;

			case INT_Ver1:	
				
				printf("\n Interrupt mode data Read\n");
				pISR_SDI_1=(unsigned)HS_READ_INT;
				
				SetBlockSizeReg(7, 512); // Maximum DMA Buffer Size, Block Size
				SetBlockCountReg(BlockNum_HSMMC); // Block Numbers to Write
				SetArgumentReg(StartAddr*512); // Card Address to Write

				if(BlockNum_HSMMC == 1)//single block
					{
						SetTransferModeReg(0, 1, 0, 1, 0);
						SetCommandReg(17, 0); // CMD17: Single-Read	
					}
				else//multi block
					{
						SetTransferModeReg(1, 1, 1, 1, 0);
						SetCommandReg(18, 0); // CMD18: Multi-Read						
					}
				
				if (!WaitForCommandComplete());
				ClearCommandCompleteStatus();

				printf("\nInterupt mode Read Start\n");
				rINTMSK &= ~(BIT_SDI1);
				rHM_NORINTSIGEN = rHM_NORINTSIGEN & ~(0xffff) | BUFFER_READREADY_SIG_INT_EN;	
				
				while(!READINT_DONE);
				
				if(!WaitForTransferComplete())
					{
					printf(("Transfer NOT Complete\n"));
					}
				ClearTransferCompleteStatus();

				printf("\nInterupt mode Read End\n");
				
	    			break;
					
			case DMA_Ver1:

				pISR_SDI_1=(unsigned)HS_DMA_INT;
				rINTMSK &= ~(BIT_SDI1);
				rHM_NORINTSTSEN &= ~(DMA_STS_INT_EN|BLOCKGAP_EVENT_STS_INT_EN);
				rHM_NORINTSIGEN = rHM_NORINTSIGEN & ~(0xffff) | TRANSFERCOMPLETE_SIG_INT_EN;
				
				SetSystemAddressReg(SDI_Rx_buffer_HSMMC);// AHB System Address For Write
				SetBlockSizeReg(7, 512); // Maximum DMA Buffer Size, Block Size				
				SetBlockCountReg(BlockNum_HSMMC); // Block Numbers to Write	
				SetArgumentReg(StartAddr*512);// Card Start Block Address to Write

				if (BlockNum_HSMMC == 1)
				{
					SetTransferModeReg(0, 1, 0, 1, 1);
					SetCommandReg(17, 0); // CMD17: Single-Read
				}
				else
				{	// (u32 MultiBlk, u32 DataDirection, u32 AutoCmd12En, u32 BlockCntEn, u32 DmaEn)
					SetTransferModeReg(1, 1, 1, 1, 1);
					SetCommandReg(18, 0); // CMD18: Multi-Read
				}

				if (!WaitForCommandComplete())
				{
					printf(("Command NOT Complete\n"));
				}
				else
					ClearCommandCompleteStatus();		
					
				while(!HS_DMA_END);				
				printf(("\nDMA Read End\n"));
				
				break;	

			default : break;			
				
		}
	
	Rx_buffer_HSMMC = (U32 *)SDI_Rx_buffer_HSMMC;
	for(j=0 ; j<(OneBlockSize*BlockNum_HSMMC)/4 ; j++)
	   	{
		      	if(j%4 == 0)
		   		printf("\n0x%04xh : ",Addr_temp);
		       	printf("0x%08x ",*Rx_buffer_HSMMC++);
		       	Addr_temp += 4;
		}
	
	printf("\n");    

	HS_DMA_END = 0;	
	BlockNum_HSMMC = 0;
	rd_cnt_HSMMC = 0;
	ReadBlockCnt_INT = 0;
}

void HS_MMC_EraseBlock(void)
{
	U32 StartAddr, EndAddr;	
	U16 EraseBlockStartNum, EraseBlockEndNum;
	
	printf("\nHigh speed MMC block erase test\n");

	printf("\nInput Erase Start Block Number(ex 0, 1, 2 ...) : ");
	EraseBlockStartNum = GetIntNum();

	printf("Input Erase End Block Number(ex 0, 1, 2 ...) : ");
	EraseBlockEndNum = GetIntNum();
	
	if(ThisIsMmc == 0)
		{
		SetArgumentReg(EraseBlockStartNum*512); 
		SetCommandReg(32, 0); 	

		while (!WaitForCommandComplete());
		ClearCommandCompleteStatus();
		
		SetArgumentReg(EraseBlockEndNum*512); 
		SetCommandReg(33, 0); 	

		while (!WaitForCommandComplete());
		ClearCommandCompleteStatus();
		}
	else if(ThisIsMmc == 1)
		{
		SetArgumentReg(EraseBlockStartNum*512); 
		SetCommandReg(35, 0); 

		while (!WaitForCommandComplete());
		ClearCommandCompleteStatus();
		
		SetArgumentReg(EraseBlockEndNum*512);
		SetCommandReg(36, 0); 

		while (!WaitForCommandComplete());
		ClearCommandCompleteStatus();
		}

	SetArgumentReg(0); 
	SetCommandReg(38, 0);

	printf("\n[SD/TFlash card Erase Block Test...End]\n");
	

}

void HS_MMC_CardDetect(void)
{
	uint16 i;

	rGPJCON = (rGPJCON & ~(0x3<<28)) | (1<<29); 
	rHM_NORINTSTSEN |= (1<<7)|(1<<6); //insert/remove interrupt enable
	rHM_NORINTSIGEN|= (1<<7)|(1<<6); //insert/remove interrupt enable
	if(rHM_PRNSTS & (1<<16))
		{
		HS_CARD_DETECT = 1;
		printf("\nCard is Inserted\n");
		}

	while(!HS_CARD_DETECT)
		{
			printf("\nInsert a Card\n");
			HS_CARD_DETECT=0;			
			HS_MMC_Irq();
		    	do {		
			if(i++ > 8) {
			
				i = 0;
				printf("\b\b\b\b\b\b\b\b\b");
			} else {
			
				printf("#");
			}					
			
			Delay(3000);
			} while(HS_CARD_DETECT==0);
		}	
	
	HS_CARD_DETECT = 0;
	rHM_CONTROL2 &= ~(1<<13);
}


int SetDataTransferWidth(void)
{
	U8  ucSfr=0;
	U8 uBitMode=0;
	U32 uArg=0;
	U8 m_ucHostCtrlReg = 0;
	U32 BD_BUS_WIDTH, ucBusWidth;

	printf("\nSelect the bus width 8-bit, 4-bit, 1-bit\n");
	printf("\nType a bus width:  ");
	BD_BUS_WIDTH = GetIntNum();

	switch (BD_BUS_WIDTH)
	{
		case 8:
			ucBusWidth = ThisIsMmc ? 8 : 4;
			break;
		case 4:
			ucBusWidth = 4;
			break;
		case 1:
			ucBusWidth = 1;
			break;
		default :
			ucBusWidth = 4;
			break;
	}

	SetSdhcCardIntEnable(0); // Disable sd card interrupt

	if(!ThisIsMmc)// <------------------------- SD Card Case
	{
		if (!IssueCommand(55, m_uRca, 0))
			return 0;
		else
		{
			if (ucBusWidth==1)
			{
				uBitMode = 0;
				if (!IssueCommand(6, 0, 1)) // 1-bits
					return 0;
			}
			else
			{
				uBitMode = 1;
				if (!IssueCommand(6, 2, 1)) // 4-bits
					return 0;
			}
		}
	}
	else // <-------------------------------- MMC Card Case
	{
		if (m_ucMMCSpecVer==4) // It is for a newest MMC Card
		{
			if (ucBusWidth==1)
				uBitMode = 0;
			else if (ucBusWidth==4)
				uBitMode = 1;//4            		// 4-bit bus
			else
				uBitMode = 2;//8-bit bus
			
			uArg=((3<<24)|(183<<16)|(uBitMode<<8));
			while(!IssueCommand(6, uArg, 0));
		}
		else
			uBitMode = 0;
	}
	
	if (uBitMode==2)
	{
		m_ucHostCtrlReg &= 0xdf;
		m_ucHostCtrlReg |= 1<<5;
	}
	else
	{
		m_ucHostCtrlReg &= 0xfd;
		m_ucHostCtrlReg |= uBitMode<<1;
	}
	
	rHM_HOSTCTL = m_ucHostCtrlReg;
	SetSdhcCardIntEnable(1);
	//printf(" transfer rHM_HOSTCTL = %x\n",rHM_HOSTCTL);
	return 1;
}

void SetSdhcCardIntEnable(U8 ucTemp)
{
    	rHM_NORINTSTSEN &= 0xFEFF;
	rHM_NORINTSTSEN |= (ucTemp<<8);
}

int SetSDOCR(void)
{
	U32 i, OCR;
	
	for(i=0; i<250; i++)
	{
	#if 0
		IssueCommand(55, 0x0000, 0); // CMD55 (For ACMD)
		IssueCommand(41, 0x0ff8000, 1); // (Ocr:2.7V~3.6V)
	#else
		IssueCommand(55, 0x0, 0); // CMD55 (For ACMD)
		IssueCommand(41, 0x0, 1); // (Ocr:2.7V~3.6V)
		OCR = rHM_RSPREG0;

		//Delay(1000);

		IssueCommand(55, 0x0, 0); // CMD55 (For ACMD)
		IssueCommand(41, OCR, 1); // (Ocr:2.7V~3.6V)		
		
	#endif
		if (rHM_RSPREG0&(0x1<<31))
		{
			//printf("\nrHM_RSPREG0=%x",rHM_RSPREG0);
			if(OCR & (1<<7))
				printf("\nVoltage range : 1.65V ~ 1.95V\n\n");
			if(OCR & (1<<21))
				printf("\nVoltage range: 2.7V ~ 3.4V\n\n");	
			else if(OCR & (1<<20))
				printf("\nVoltage range: 2.7V ~ 3.3V\n\n");	
			else if(OCR & (1<<19))
				printf("\nVoltage range: 2.7V ~ 3.2V\n\n");	
			else if(OCR & (1<<18))
				printf("\nVoltage range: 2.7V ~ 3.1V\n\n");	
			ThisIsMmc = 0;
			return 1;
		}
		Delay(1);
	}
	// The current card is MMC card, then there's time out error, need to be cleared.
	ClearErrInterruptStatus();
	return 0;
}

int SetMMCOCR(void)
{
	U32 i, OCR;
	for (i=0; i<250; i++)
	{
		IssueCommand(1, 0x0, 0);
		//printf("\nCMD 1 RSP = %x\n",rHM_RSPREG0);
		OCR = rHM_RSPREG0;	
		
		IssueCommand(1, OCR, 0); // (Ocr:2.7V~3.6V)
		if (rHM_RSPREG0&(0x1<<31))
		{
			if(OCR & (1<<7))
				printf("\nVoltage range : 1.65V ~ 1.95V\n\n");
			if(OCR & (1<<21))
				printf("\nVoltage range: 2.7V ~ 3.4V\n\n");	
			else if(OCR & (1<<20))
				printf("\nVoltage range: 2.7V ~ 3.3V\n\n");	
			else if(OCR & (1<<19))
				printf("\nVoltage range: 2.7V ~ 3.2V\n\n");	
			else if(OCR & (1<<18))
				printf("\nVoltage range: 2.7V ~ 3.1V\n\n");	
			ThisIsMmc=1;
			return 1;
		}
	}
	// The current card is SD card, then there's time out error, need to be cleared.
	ClearErrInterruptStatus();
	return 0;
}

void SetCommandReg(U16 uCmd,U32 uIsAcmd)
{
	U16 usSfr = 0;
	
	//rHM_CMDREG &= ~(0xffff);
	
	if (!uIsAcmd)//No ACMD
	{
		/* R2: 136-bits Resp.*/
		if ((uCmd==2||uCmd==9||uCmd==10))
			usSfr=(uCmd<<8)|(0<<4)|(1<<3)|(1<<0);

		/* R1,R6,R5: 48-bits Resp. */
		else if (uCmd==3||uCmd==13||uCmd==16||uCmd==27||uCmd==30||uCmd==32||uCmd==33||uCmd==35||uCmd==36||uCmd==42||uCmd==55||uCmd==56)
			usSfr=(uCmd<<8)|(1<<4)|(1<<3)|(2<<0);

		else if (uCmd==11||uCmd==14||uCmd==17||uCmd==18||uCmd==19||uCmd==20||uCmd==24||uCmd==25)
			usSfr=(uCmd<<8)|(1<<5)|(1<<4)|(1<<3)|(2<<0);

		/* R1b,R5b: 48-bits Resp. */
		else if (uCmd==6||uCmd==7||uCmd==12||uCmd==28||uCmd==29||uCmd==38)
		{
			if (uCmd==12)
				usSfr=(uCmd<<8)|(3<<6)|(1<<4)|(1<<3)|(3<<0);
			else if (uCmd==6)
			{
				if(!ThisIsMmc)	// SD card
					usSfr=(uCmd<<8)|(1<<5)|(1<<4)|(1<<3)|(2<<0);
				else			// MMC card
					usSfr=(uCmd<<8)|(1<<4)|(1<<3)|(3<<0);
					//usSfr=(uCmd<<8)|(1<<5)|(1<<4)|(1<<3)|(3<<0);
			}
			else
				usSfr=(uCmd<<8)|(1<<4)|(1<<3)|(3<<0);
		}

		/* R3,R4: 48-bits Resp.  */
		else if (uCmd==1)
			usSfr=(uCmd<<8)|(0<<4)|(0<<3)|(2<<0);

		/* No-Resp. */
		else
			usSfr=(uCmd<<8)|(0<<4)|(0<<3)|(0<<0);
	}
	else//ACMD
	{
		if (uCmd==6||uCmd==22||uCmd==23)		// R1
			usSfr=(uCmd<<8)|(1<<4)|(1<<3)|(2<<0);
		else if (uCmd==13||uCmd==51)
			usSfr=(uCmd<<8)|(1<<5)|(1<<4)|(1<<3)|(2<<0);
		else
			usSfr=(uCmd<<8)|(0<<4)|(0<<3)|(2<<0);
	}
	rHM_CMDREG = usSfr;
}


void SetMMCSpeedMode(U32 eSDSpeedMode)
{
	U8  ucSpeedMode;
	U32 uArg=0;
	ucSpeedMode = (eSDSpeedMode == HIGH) ? 1 : 0;
	uArg=(3<<24)|(185<<16)|(ucSpeedMode<<8); // Change to the high-speed mode
	while(!IssueCommand(6, uArg, 0));	
}


void SetSDSpeedMode(U32 eSDSpeedMode)
{
	U32 uSfr;
	U32 uArg = 0;
	U8  ucSpeedMode;
	int i;
	
	ucSpeedMode = (eSDSpeedMode == HIGH) ? 1 : 0;

	if (!IssueCommand(16, 64, 0)) // CMD16
		printf("CMD16 fail\n");
	else
	{
		SetBlockSizeReg(7, 64);
		SetBlockCountReg(1);
		SetArgumentReg(0*64);

		SetTransferModeReg(0, 1, 0, 0, 0);

		uArg = (0x1<<31)|(0xffff<<8)|(ucSpeedMode<<0);

		if (!IssueCommand(6, uArg, 0))
			printf("CMD6 fail\n");
		else
		{
			WaitForBufferReadReady();
			ClearBufferReadReadyStatus();
			
			for(i=0; i<16; i++)
			{
				uSfr = rHM_BDATA	;			
			}			

			WaitForTransferComplete();
			ClearTransferCompleteStatus();
		}
	}	
}


void SetClock(U32 ClkSrc, U16 Divisor)
{

	rHM_CONTROL2 = (rHM_CONTROL2 & ~(0xffffffff)) | (0x1<<15)|(0x1<<14)|(0x1<<8)|(ClkSrc<<4);
	//rHM_CONTROL2 = (rHM_CONTROL2 & ~(0xffffffff)) |(0x1<<14)|(0x1<<8)|(ClkSrc<<4);
	rHM_CONTROL3 = (0<<31) | (1<<23) | (0<<15) | (1<<7);

	// SDCLK Value Setting + Internal Clock Enable	
	rHM_CLKCON = (rHM_CLKCON & ~((0xff<<8)|(0x1))) | (Divisor<<8)|(1<<0);

	// CheckInternalClockStable
	while (!(rHM_CLKCON&0x2));
	ClockOnOff(1);
	printf("rHM_CONTROL2 = %x\n",rHM_CONTROL2);
	printf("rHM_CLKCON = %x\n",rHM_CLKCON);
}

int IssueCommand( U16 uCmd, U32 uArg, U32 uIsAcmd)
{
	U32 uSfr;

	while ((rHM_PRNSTS&0x1)); // Check CommandInhibit_CMD

	if (!uIsAcmd)//R1b type commands have to check CommandInhibit_DAT bit
	{
		if((uCmd==6&&ThisIsMmc)||uCmd==7||uCmd==12||uCmd==28||uCmd==29||uCmd==38||((uCmd==42||uCmd==56)&&(!ThisIsMmc)))
		{
			do	{
				uSfr = rHM_PRNSTS;
			} while((uSfr&0x2)); // Check CommandInhibit_DAT
		}
	}
	// Argument Setting
	if (!uIsAcmd)// <------------------- Normal Command (CMD)
	{
		if(uCmd==3||uCmd==4||uCmd==7||uCmd==9||uCmd==10||uCmd==13||uCmd==15||uCmd==55)
			rHM_ARGUMENT = uArg<<16;
		else
			rHM_ARGUMENT = uArg;
	}
	else// <--------------------------- APP.Commnad (ACMD)
		rHM_ARGUMENT = uArg;

	SetCommandReg(uCmd,uIsAcmd);

	if (!WaitForCommandComplete())
		{
			printf(("Command NOT Complete\n"));
		}
	else
		ClearCommandCompleteStatus();	

	if (!(rHM_NORINTSTS&0x8000))
	{
		return 1;
	}
	else
	{
		if(OCRcheck == 1)
			return 0;
		else
			{
			printf("Command = %d, Error Stat = %x\n",(rHM_CMDREG>>8),rHM_ERRINTSTS);		
			return 0;								
			}
	}

}

void GetResponseData(U32 uCmd)
{
	U32 uSfr0,uSfr1,uSfr2,uSfr3;

	uSfr0 = rHM_RSPREG0;
	uSfr1 = rHM_RSPREG1;
	uSfr2 = rHM_RSPREG2;
	uSfr3 = rHM_RSPREG3;

	if(uCmd==3)
	{
 		if(!ThisIsMmc)
 		{
			m_uRca = (uSfr0>>16)&0xFFFF;			
			printf("=>  RCA=%d\n", m_uRca);
 		}
	}
	else if (uCmd==9)
	{
		if(ThisIsMmc)
		{
			m_ucMMCSpecVer=(rHM_RSPREG3>>18)& 0xF;
			//m_ucMMCSpecVer=(uSfr3>>26)& 0xF;
			printf("=>  m_ucMMCSpecVer=%d\n", m_ucMMCSpecVer);
		}
	}
}

int GetSdSCR()
{
	U32  uSCR1, uSCR2;

	if (!IssueCommand(16, 8, 0))
		return 0;
	else
	{
		SetBlockSizeReg(7, 8);
		SetBlockCountReg(1);
		SetArgumentReg(0*8);

		SetTransferModeReg(0, 1, 0, 0, 0);
		if (!IssueCommand(55, m_uRca, 0))		// CMD55 (For ACMD)
			return 0;
		else
		{
			if (!IssueCommand(51, 0, 1))			// Acmd51
				return 0;
			else
			{
				WaitForBufferReadReady();
				ClearBufferReadReadyStatus();

				uSCR1 = rHM_BDATA;
				uSCR2 = rHM_BDATA;

				WaitForTransferComplete();
				ClearTransferCompleteStatus();
/*
				printf("\nSCR = %d\n",(uSCR1>>24)&0xf);
				printf("\nSCR = %d\n",(uSCR2>>24)&0xf);

				if ((uSCR2>>24)&0xf)
					SDSpecVer = 1; // Version 1.10, support cmd6
				else
					SDSpecVer = 0; // Version 1.0 ~ 1.01
*/

				//if (uSCR1&0x1)
					SDSpecVer = 1; // Version 1.10, support cmd6
				//else
				//	SDSpecVer = 0; // Version 1.0 ~ 1.01

				printf("              =>  m_ucSDSpecVer=%d\n", SDSpecVer);
				return 1;
			}
		}
	}
}

int IsCardInProgrammingState(void)
{
	// check the card status.
	U32  uSfr;

	if (!IssueCommand(13, m_uRca, 0))
		return 0;
	else
	{
		if(((rHM_RSPREG0>>9)&0xf) == 4)
			{
			printf("Card is transfer status\n");
			return 1;
			}
	}
}

void HostCtrlSpeedMode(U8 SpeedMode)
{
	U8  ucSpeedMode;
	ucSpeedMode = (SpeedMode == HIGH) ? 1 : 0;
	rHM_HOSTCTL &= ~(0x1<<2);
	rHM_HOSTCTL |= ucSpeedMode<<2;	
}

#if NEW_SD_CARD

void ClockConfig(U32 Clksrc, U32 Divisior)
{
	U32 SrcFreq, WorkingFreq;
	U32 RealDivisor;
	//printf("Clock Config\n");
	
	if (Clksrc == SD_HCLK)
		SrcFreq = HCLK;
	else if (Clksrc == SD_EPLL)//Epll Out 48MHz
		SrcFreq = 100000000;
	else
		Clksrc = HCLK;

	WorkingFreq = SrcFreq/(Divisior*2);
	printf("WorkingFreq = %dMHz\n",WorkingFreq/(1000000));

	if (ThisIsMmc)
	{
		if (m_ucMMCSpecVer==4)
		{
			if (WorkingFreq>20000000)// It is necessary to enable the high speed mode in the card before changing the clock freq to a freq higher than 20MHz.
			{
				SetMMCSpeedMode(HIGH);					
				printf("\nSet MMC High speed mode OK!!\n");
			}
			else
			{
				SetMMCSpeedMode(NORMAL);	
				printf("\nSet MMC Normal speed mode OK!!\n");
			}
		}
		else // old version
			printf("Old version MMC card can not support working frequency higher than 25MHz");
	}
	else
	{
		GetSdSCR();
		if (SDSpecVer==1)
		{
			if (WorkingFreq>25000000)
			{
				SetSDSpeedMode(HIGH);//Higher than 25MHz, should use high speed mode. Max 50MHz and 25MB/sec
				printf("\nSet SD High speed mode OK!!\n");
			}
			else
			{
				SetSDSpeedMode(NORMAL);
				printf("\nSet SD Normal speed mode OK!!\n");
			}
		}
		else 
			printf("Old version SD card can not support working frequency higher than 25MHz");
	}

	if (WorkingFreq>25000000)// Higher than 25MHz, it is necessary to enable high speed mode of the host controller.
	{
		HostCtrlSpeedMode(HIGH);
	}
	else
	{
		HostCtrlSpeedMode(NORMAL);
	}

	ClockOnOff(0); // when change the sd clock frequency, need to stop sd clock.
	SetClock(Clksrc, Divisior); 
	printf("clock config rHM_HOSTCTL = %x\n",rHM_HOSTCTL);
		
}
#else

void ClockConfig(U32 Clksrc, U32 Divisior)
{
	U32 SrcFreq, WorkingFreq;

	//printf("Clock Config\n");
	
	if (Clksrc == SD_HCLK)
		SrcFreq = HCLK;
	else if (Clksrc == SD_EPLL)//Epll Out 48MHz
		SrcFreq = 100000000;
	else
		Clksrc = HCLK;

	WorkingFreq = SrcFreq/(Divisior*2);
	printf("Card Working Frequency = %dMHz\n",WorkingFreq/(1000000));

	if (ThisIsMmc)
	{
		if (m_ucMMCSpecVer==4)
		{
			if (WorkingFreq>20000000)// It is necessary to enable the high speed mode in the card before changing the clock freq to a freq higher than 20MHz.
			{
				SetMMCSpeedMode(HIGH);					
				printf("\nSet MMC High speed mode OK!!\n");
			}
			else
			{
				SetMMCSpeedMode(NORMAL);	
				printf("\nSet MMC Normal speed mode OK!!\n");
			}
		}
		else // old version
			printf("Old version MMC card can not support working frequency higher than 25MHz");
	}

	if (WorkingFreq>25000000)// Higher than 25MHz, it is necessary to enable high speed mode of the host controller.
	{
		HostCtrlSpeedMode(HIGH);
	}
	else
	{
		HostCtrlSpeedMode(NORMAL);
	}

	ClockOnOff(0); // when change the sd clock frequency, need to stop sd clock.
	SetClock(Clksrc, Divisior); 
	printf("clock config rHM_HOSTCTL = %x\n",rHM_HOSTCTL);
		
}
#endif

void HS_MMC_Irq(void)
{
	pISR_SDI_1 = (int)HS_CARD_DETECT_INT;	// Set interrupt service routine.	
	rINTMSK &= ~(BIT_SDI1);
}

void HS_MMC_Reset(void)
{
	rHM_SWRST = 0x3;
}

void HS_MMC_SETGPIO(void)
{
	rGPLCON = rGPLCON & ~(0xfffff) | (0xaaaaa);
	rGPJCON = rGPJCON & ~(0x3f<<26) | (0x2a<<26);
	//rGPJCON &= ~(0x3<<30) | (1<<30);
	//rGPJDAT |=(1<<15);
}

void SetTransferModeReg(U32 MultiBlk,U32 DataDirection, U32 AutoCmd12En,U32 BlockCntEn,U32 DmaEn)
{
	rHM_TRNMOD = (rHM_TRNMOD & ~(0xffff)) | (MultiBlk<<5)|(DataDirection<<4)|(AutoCmd12En<<2)|(BlockCntEn<<1)|(DmaEn<<0);
	printf("\nrHM_TRNMOD = %x\n",rHM_TRNMOD);
}

void SetArgumentReg(U16 uArg)
{
	rHM_ARGUMENT = uArg;
}

void SetBlockCountReg(U16 uBlkCnt)
{
	rHM_BLKCNT = uBlkCnt;
}

void SetSystemAddressReg(U32 SysAddr)
{
	rHM_SYSAD = SysAddr;
}

void SetBlockSizeReg(U16 uDmaBufBoundary, U16 uBlkSize)
{
	rHM_BLKSIZE = (uDmaBufBoundary<<12)|(uBlkSize);
}

void ClockOnOff(int OnOff)
{
	if (OnOff == 0)
	{
		rHM_CLKCON &=~(0x1<<2);

		while (1)
		{			
			if (!(rHM_CLKCON&(0x1<<3))) 	// SD_CLKSRC is Stable
				break;
		}

	}
		
	else
	{
		rHM_CLKCON|=(0x1<<2);		
		while (1)
		{
			if (rHM_CLKCON&(0x1<<3)) // SD_CLKSRC is Stable
				break;
		}
	}
}

void ClearCommandCompleteStatus(void)
{
	rHM_NORINTSTS=(1<<0);
	while (rHM_NORINTSTS&0x1)
		{
			rHM_NORINTSTS=(1<<0);
		}	
}

void ClearTransferCompleteStatus(void)
{

	rHM_NORINTSTS |= (1<<1);
	while (rHM_NORINTSTS&0x2)
		{
			rHM_NORINTSTS |= (1<<1);
		}
}

void ClearBufferWriteReadyStatus(void)
{
	rHM_NORINTSTS |= (1<<4);
	while (rHM_NORINTSTS & 0x10)
		rHM_NORINTSTS |= (1<<4);
}

void ClearBufferReadReadyStatus(void)
{
	rHM_NORINTSTS |= (1<<5);
	while (rHM_NORINTSTS & 0x20)
		rHM_NORINTSTS |= (1<<5);
}


void ClearErrInterruptStatus(void)
{
	while (rHM_NORINTSTS&(0x1<<15))
	{
		rHM_NORINTSTS =rHM_NORINTSTS;
		rHM_ERRINTSTS =rHM_ERRINTSTS;
	}
}

int WaitForBufferWriteReady(void)
{
	U32 uLoop=0;

	while (!(rHM_NORINTSTS&0x10))
	{
		if (uLoop%500000==0&&uLoop>0)
		{			
			return 0;
		}
		uLoop++;
	}
	return 1;
}

int WaitForBufferReadReady(void)
{
	U32 uLoop=0;

	while (!(rHM_NORINTSTS&0x20))
	{
		if (uLoop%500000==0&&uLoop>0)
		{			
			return 0;
		}
		uLoop++;
	}
	return 1;
}

int WaitForCommandComplete(void)
{
	U32 Loop=0;

	while (!(rHM_NORINTSTS&0x1))
	{
		if (Loop%500000==0&&Loop>0)
		{			
			return 0;
		}
		Loop++;
	}
	return 1;
}

int WaitForTransferComplete(void)
{
	U32 Loop=0;
/*
	while (!(rHM_NORINTSTS&0x2))
	{
		if (Loop%500000==0&&Loop>0)
		{			
			return 0;
		}
		Loop++;
	}
*/
	while (!(rHM_NORINTSTS&0x2))
	return 1;
}

void InterruptEnable(U16 NormalIntEn, U16 ErrorIntEn)
{
	ClearErrInterruptStatus();	
	rHM_NORINTSTSEN = NormalIntEn;
	rHM_ERRINTSTSEN = ErrorIntEn;
}

#if BUFFER_BOUNDARY
void __irq HS_DMA_INT(void)
{
	U32 i;

    	ClearPending(BIT_SDI1);
	rINTMSK |= (BIT_SDI1);
	BufferBoundary_INT_Cnt++;
	//printf("\nISR rHM_NORINTSTS = %x",rHM_NORINTSTS);
	
	if(rHM_NORINTSTS & (1<<1))
	{
		//printf("\nTransfer Complete\n");
		HS_DMA_END=1;
		rINTMSK |= (BIT_SDI1);		
	}
	else if(rHM_NORINTSTS & (1<<3))
	{
		rHM_NORINTSTS |= (1<<3);
		i = BufferBoundary_INT_Cnt * 0x1000;// 4K Byte boundary
		SetSystemAddressReg(SDI_Tx_buffer_HSMMC + i);
		rINTMSK &= ~(BIT_SDI1);	
	}

}
#else
void __irq HS_DMA_INT(void)
{
    	ClearPending(BIT_SDI1);
	rINTMSK |= (BIT_SDI1);
	printf("\nISR rHM_NORINTSTS = %x",rHM_NORINTSTS);
	if(rHM_NORINTSTS & (1<<1))
	{
		//printf("\nTransfer Complete\n");
		HS_DMA_END=1;	
		rHM_NORINTSTS |= (1<<1);
	}
}
#endif
void __irq HS_WRITE_INT(void)
{
	int i;

	//printf("\nWrite ISR");
	
    	ClearPending(BIT_SDI1);
	rINTMSK |= (BIT_SDI1);	
/*
	for(j=0; j<BlockNum_HSMMC; j++)
	{
		if (!WaitForBufferWriteReady())
			printf("WriteBuffer NOT Ready\n");
		else
			ClearBufferWriteReadyStatus();

		for(i=0; i<512/4; i++)
			{
				rHM_BDATA = *Tx_buffer_HSMMC++;	
  		 		wt_cnt_HSMMC++;						
			}
		WriteBlockCnt ++;
		printf("\nWrite block count = %d", WriteBlockCnt);
	}
*/
	if (!WaitForBufferWriteReady())
		printf("WriteBuffer NOT Ready\n");
	else
		ClearBufferWriteReadyStatus();

	for(i=0; i<512/4; i++)
	{
		rHM_BDATA = *Tx_buffer_HSMMC++;	
	 	wt_cnt_HSMMC++;						
	}
	
	WriteBlockCnt_INT ++;
	//printf("\nWrite block count = %d", WriteBlockCnt_INT);	

	rINTMSK &= ~(BIT_SDI1);
	
	if(BlockNum_HSMMC ==  WriteBlockCnt_INT)
		{
		WRITEINT_DONE = 1;
		ClearPending(BIT_SDI1);
		rINTMSK |= (BIT_SDI1);
		}	
}

void __irq HS_READ_INT(void)
{
	int i;

	printf("\nWrite ISR");
	
    	ClearPending(BIT_SDI1);
	rINTMSK |= (BIT_SDI1);	
/*
	for(j=0; j<BlockNum_HSMMC; j++)
	{
		if (!WaitForBufferWriteReady())
			printf("WriteBuffer NOT Ready\n");
		else
			ClearBufferWriteReadyStatus();

		for(i=0; i<512/4; i++)
			{
				rHM_BDATA = *Tx_buffer_HSMMC++;	
  		 		wt_cnt_HSMMC++;						
			}
		WriteBlockCnt ++;
		printf("\nWrite block count = %d", WriteBlockCnt);
	}
*/
	if (!WaitForBufferReadReady())
		printf("WriteBuffer NOT Ready\n");
	else
		ClearBufferReadReadyStatus();

	for(i=0; i<512/4; i++)
	{
		*Rx_buffer_HSMMC++ = rHM_BDATA;
		rd_cnt_HSMMC++;
	}
	
	ReadBlockCnt_INT ++;
	printf("\nWrite block count = %d", ReadBlockCnt_INT);	

	rINTMSK &= ~(BIT_SDI1);
	
	if(BlockNum_HSMMC ==  ReadBlockCnt_INT)
		{
		READINT_DONE = 1;
		ClearPending(BIT_SDI1);
		rINTMSK |= (BIT_SDI1);
		}	
}

void __irq HS_READ_COMPARE_INT(void)
{
	int i;

	printf("\nWrite ISR");
	
    	ClearPending(BIT_SDI1);
	rINTMSK |= (BIT_SDI1);	

	if (!WaitForBufferReadReady())
		printf("WriteBuffer NOT Ready\n");
	else
		ClearBufferReadReadyStatus();

	for(i=0; i<512/4; i++)
	{
		*Compare_buffer_HSMMC++ = rHM_BDATA;
	}
	
	CompareCnt_INT ++;
	printf("\nWrite block count = %d", CompareCnt_INT);	

	rINTMSK &= ~(BIT_SDI1);
	
	if(BlockNum_HSMMC ==  CompareCnt_INT)
		{
		COMPARE_INT_DONE = 1;
		ClearPending(BIT_SDI1);
		rINTMSK |= (BIT_SDI1);
		}	
}

void __irq HS_CARD_DETECT_INT(void)
{
    	ClearPending(BIT_SDI1);
	rINTMSK |= (BIT_SDI1);
	rHM_NORINTSIGEN &= ~((1<<7)|(1<<6));//Normal interrupt signal enable
	
	printf("Card Detect ISR\n");

	if(rHM_PRNSTS & (1<<16))
		{
		printf("\nCard insert\n");
		HS_CARD_DETECT = 1;
		}
	else
		{
		printf("\nCard removal\n");		
		HS_CARD_DETECT = 1;
		}
	
	rHM_NORINTSTS |= (1<<7)|(1<<6);
	rINTMSK &= ~(BIT_SDI1);
}

void DisplayCardInformation(void)
{
	U32 C_SIZE, C_SIZE_MULT, READ_BL_LEN, READ_BL_PARTIAL, CardSize, OneBlockSize;
	
	if(ThisIsMmc)
	{
		m_ucMMCSpecVer=(rHM_RSPREG3>>18)& 0xF;
		printf("=>  m_ucMMCSpecVer=%d\n", m_ucMMCSpecVer);
	}

	READ_BL_LEN = ((rHM_RSPREG2>>8) & 0xf) ;
	READ_BL_PARTIAL = ((rHM_RSPREG2>>7) & 0x1) ;
	C_SIZE = ((rHM_RSPREG2 & 0x3) << 10) | ((rHM_RSPREG1 >> 22) & 0x3ff);
	C_SIZE_MULT = ((rHM_RSPREG1>>7)&0x7);
	
	CardSize = (1<<READ_BL_LEN)*(C_SIZE+1)*(1<<(C_SIZE_MULT+2))/1048576;
	OneBlockSize = (1<<READ_BL_LEN);

	printf("\n READ_BL_LEN: %d",READ_BL_LEN);	
	printf("\n READ_BL_PARTIAL: %d",READ_BL_PARTIAL);	
	printf("\n C_SIZE: %d",C_SIZE);	
	printf("\n C_SIZE_MULT: %d\n",C_SIZE_MULT);	

	printf("\n One Block Size: %dByte",OneBlockSize);	
	printf("\n Total Card Size: %dMByte\n\n\n",CardSize+1);	
}

void CalculationBPS_HSMMC(int Time)
{
	float x=0;
	int y=0;
	float bps=0;

	x = (float)((float)1000000/(float)Time); //Unit is usec

	y = BlockNum_HSMMC * 512 * 8;
	
	bps = x*(float)y;
	
	printf("\n\n\nTransfer Time = %dusec",Time);
	printf("\nTransferSize = %dKByte",y/(8*1024));
	printf("\nBPS = %fMByte/sec\n\n",bps/(1000000*8));
}

void DataRead_ForCompare(int StartAddr)
{
	U32 i=0,j=0;
	COMPARE_INT_DONE = 0;

	ClearPending(BIT_SDI1);
	rHM_NORINTSIGEN &= ~(0xffff);
	
	Compare_buffer_HSMMC = (U32 *)SDI_Compare_buffer_HSMMC;
    	for(i=0 ; i<(512 * BlockNum_HSMMC)/4 ; i++)	
		*(Compare_buffer_HSMMC+i) = 0x0;   
		
	printf("\nPolling mode data read\n");
	printf("\nRead BlockNum = %d\n",BlockNum_HSMMC);

	while (!IsCardInProgrammingState());

	SetBlockSizeReg(7, 512); // Maximum DMA Buffer Size, Block Size
	SetBlockCountReg(BlockNum_HSMMC); // Block Numbers to Write
	SetArgumentReg(StartAddr*512); // Card Address to Write

	if(BlockNum_HSMMC == 1)//single block
		{
			printf("Single block read\n");
			SetTransferModeReg(0, 1, 1, 1, 0);
			SetCommandReg(17, 0); // CMD17: Single-Read						
		}
	else//multi block
		{
			printf("Multi block read\n");
			SetTransferModeReg(1, 1, 1, 1, 0);
			SetCommandReg(18, 0); // CMD18: Multi-Read
		}
	
	if (!WaitForCommandComplete())
	{
		printf("\nCommand is NOT completed\n");
	}
	ClearCommandCompleteStatus();	
	
	for(j=0; j<BlockNum_HSMMC; j++)
		{
			if (!WaitForBufferReadReady());
				//printf("ReadBuffer NOT Ready\n");
			else
				ClearBufferReadReadyStatus();
			for(i=0; i<512/4; i++)
			{
				*Compare_buffer_HSMMC++ = rHM_BDATA;
				CompareCnt_INT++;
			}
		}

	printf("\nRead count=%x\n",CompareCnt_INT);
	if(!WaitForTransferComplete())
		{
		printf(("Transfer NOT Complete\n"));
		}
	ClearTransferCompleteStatus();

	printf("\n\nrHM_NORINTSTS = %x",rHM_NORINTSTS);
}

void DataCompare_HSMMC(U32 a0, U32 a1, U32 bytes)
{	
	U32 * pD0 = (U32 *)a0;
	U32 * pD1 = (U32 *)a1;
	U32  ErrCnt = 0;
	U32 i;

	for (i=0; i<bytes; i++)
	{
		if (*pD0 != *pD1) 
		{
			//printf("\n%08x=%02x <-> %08x=%02x", pD0, *pD0, pD1, *pD1);
			ErrCnt++;
		}
		pD0++;
		pD1++;
	}
	printf("\nTotal Error cnt = %d",ErrCnt);

	if(ErrCnt == 0)
		printf("\nData Compare Ok\n");
}
