/*********************************************************************
;* Project Name : S3C2443
;*
;* Copyright 2005 by Samsung Electronics, Inc.
;* All rights reserved.
;*
;* Project Description :
;* This software is only for verifying functions of the S3C2443
;* Anybody can use this code without our permission.
;**********************************************************************
;*
;*	Description	:	Camera
;*	
;*	History
;*			R0.0 (2006.8. ) : Y.C.Kwon draft
;*
;**********************************************************************/

//////////////////////////////////////
// Put this into system library or erase every 'Assert'. This is not necessary function.
#include <stdio.h>
#define Assert(condition,message) { if (!(condition)) printf(message, __FILE__, __LINE__); }
//////////////////////////////////////


#include "Camera.h"

//	Chapter23	Camera Interface
#define FIMC_BASE		(0x4d800000)
#define rCISRCFMT		(*(volatile unsigned *)(FIMC_BASE+0x00))	//Input Source Format        
#define rCIWDOFST		(*(volatile unsigned *)(FIMC_BASE+0x04))	//Window offset       
#define rCIGCTRL		(*(volatile unsigned *)(FIMC_BASE+0x08))	//Global control        
#define rCIDOWSFT2		(*(volatile unsigned *)(FIMC_BASE+0x14))	//Window option 2
#define rCICOYSA1		(*(volatile unsigned *)(FIMC_BASE+0x18))	//Y1 frame start address for codec DMA      
#define rCICOYSA2		(*(volatile unsigned *)(FIMC_BASE+0x1C))	//Y2 frame start address for codec DMA       
#define rCICOYSA3		(*(volatile unsigned *)(FIMC_BASE+0x20))	//Y3 frame start address for codec DMA        
#define rCICOYSA4		(*(volatile unsigned *)(FIMC_BASE+0x24))	//Y4 frame start address for codec DMA          
#define rCICOCBSA1		(*(volatile unsigned *)(FIMC_BASE+0x28))	//Cb1 frame start address for codec DMA 
#define rCICOCBSA2		(*(volatile unsigned *)(FIMC_BASE+0x2C))	//Cb2 frame start address for codec DMA        
#define rCICOCBSA3		(*(volatile unsigned *)(FIMC_BASE+0x30))	//Cb3 frame start address for codec DMA           
#define rCICOCBSA4		(*(volatile unsigned *)(FIMC_BASE+0x34))	//Cb4 frame start address for codec DMA   
#define rCICOCRSA1		(*(volatile unsigned *)(FIMC_BASE+0x38))	//Cr1 frame start address for codec DMA
#define rCICOCRSA2		(*(volatile unsigned *)(FIMC_BASE+0x3C))	//Cr2 frame start address for codec DMA
#define rCICOCRSA3		(*(volatile unsigned *)(FIMC_BASE+0x40))	//Cr3 frame start address for codec DMA
#define rCICOCRSA4		(*(volatile unsigned *)(FIMC_BASE+0x44))	//Cr4 frame start address for codec DMA
#define rCICOTRGFMT		(*(volatile unsigned *)(FIMC_BASE+0x48))	//Target image format of codex DMA
#define rCICOCTRL		(*(volatile unsigned *)(FIMC_BASE+0x4C))	//Codec DMA comtrol        
#define rCICOSCPRERATIO	(*(volatile unsigned *)(FIMC_BASE+0x50))	//Codec pre-scaler ratio control      
#define rCICOSCPREDST	(*(volatile unsigned *)(FIMC_BASE+0x54))	//Codec pre-scaler desitination format
#define rCICOSCCTRL		(*(volatile unsigned *)(FIMC_BASE+0x58))	//Codec main-scaler control
#define rCICOTAREA		(*(volatile unsigned *)(FIMC_BASE+0x5C))	//Codec pre-scaler desination format
#define rCICOSTATUS		(*(volatile unsigned *)(FIMC_BASE+0x64))	//Codec path status
#define rCIPRCLRSA1		(*(volatile unsigned *)(FIMC_BASE+0x6C))	//RGB 1st frame start address for preview DMA
#define rCIPRCLRSA2		(*(volatile unsigned *)(FIMC_BASE+0x70))	//RGB 1st frame start address for preview DMA
#define rCIPRCLRSA3		(*(volatile unsigned *)(FIMC_BASE+0x74))	//RGB 1st frame start address for preview DMA
#define rCIPRCLRSA4		(*(volatile unsigned *)(FIMC_BASE+0x78))	//RGB 1st frame start address for preview DMA
#define rCIPRTRGFMT		(*(volatile unsigned *)(FIMC_BASE+0x7C))	//Target image format of Preview DMA
#define rCIPRCTRL		(*(volatile unsigned *)(FIMC_BASE+0x80))	//Codec DMA comtrol        
#define rCIPRSCPRERATIO	(*(volatile unsigned *)(FIMC_BASE+0x84))	//Codec pre-scaler ratio control      
#define rCIPRSCPREDST	(*(volatile unsigned *)(FIMC_BASE+0x88))	//Codec pre-scaler desitination format
#define rCIPRSCCTRL		(*(volatile unsigned *)(FIMC_BASE+0x8C))	//Codec main-scaler control
#define rCIPRTAREA		(*(volatile unsigned *)(FIMC_BASE+0x90))	//Codec pre-scaler desination format
#define rCIPRSTATUS		(*(volatile unsigned *)(FIMC_BASE+0x98))	//Codec path status
#define rCIIMGCPT		(*(volatile unsigned *)(FIMC_BASE+0xA0))	//Imahe capture enable command
#define rCICOCPTSEQ		(*(volatile unsigned *)(FIMC_BASE+0xA4))	//Codec dma capture sequence related
#define rCICOSCOS		(*(volatile unsigned *)(FIMC_BASE+0xA8))	//Codec scan line offset related
#define rCIIMGEFF		(*(volatile unsigned *)(FIMC_BASE+0xB0))	//Imahe Effects related
#define rCIMSYSA		(*(volatile unsigned *)(FIMC_BASE+0xB4))	//MSDMA Y start address related
#define rCIMSCBSA		(*(volatile unsigned *)(FIMC_BASE+0xB8))	//MSDMA Cb start address related
#define rCIMSCRSA		(*(volatile unsigned *)(FIMC_BASE+0xBC))	//MSDMA Cr start address related
#define rCIMSYEND		(*(volatile unsigned *)(FIMC_BASE+0xC0))	//MSDMA Y end address related
#define rCIMSCBEND		(*(volatile unsigned *)(FIMC_BASE+0xC4))	//MSDMA Cb end address related
#define rCIMSCREND		(*(volatile unsigned *)(FIMC_BASE+0xC8))	//MSDMA Cr end address related
#define rCIMSYOFF		(*(volatile unsigned *)(FIMC_BASE+0xCC))	//MSDMA Y offset related
#define rCIMSCBOFF		(*(volatile unsigned *)(FIMC_BASE+0xD0))	//MSDMA Cb offset related
#define rCIMSCROFF		(*(volatile unsigned *)(FIMC_BASE+0xD4))	//MSDMA Cr offset related
#define rCIMSWIDTH		(*(volatile unsigned *)(FIMC_BASE+0xD8))	//MSDMA source image width related
#define rCIMSCTRL		(*(volatile unsigned *)(FIMC_BASE+0xDC))	//MSDMA cotrol


//*******************************************************************
//*		Functions for Camera Interface
//*******************************************************************

void ResetCameraModule( void)
{
	volatile int delay;
	
	rCIGCTRL	&=	~(1<<30);
	for(delay=0;delay<1000000;delay++);	//	substitude this with Delay() function.
	
	rCIGCTRL	|=	(1<<30);
	for(delay=0;delay<1000000;delay++);	//	substitude this with Delay() function.
}

void EnableImageCapture( void)
{
	rCIIMGCPT	|=	(1u<<31);
}

void DisableImageCapture( void)
{
	rCIIMGCPT	&=	~(1u<<31);
}

void ResetCameraInterface( void)
{
	rCISRCFMT	|=	(1u<<31);

	rCIGCTRL	|=	(1u<<31);
	rCIGCTRL	&=	~(1u<<31);
}

void SetCameraSourceFormat( unsigned int ITU601_656, unsigned int UVOffset, unsigned int Order422)
{
	rCISRCFMT	=	(rCISRCFMT & ~((0x1<<31)|(0x1<<30)|(0x3<<14))) | (ITU601_656<<31)|(UVOffset<<30)|(Order422<<14);
}

void SetCameraSourceSize( unsigned int SourceHsize, unsigned int SourceVsize)
{
	rCISRCFMT	=	(rCISRCFMT & ~((0x1fff<<16)|(0x1fff<<0))) | (SourceHsize<<16)|(SourceVsize);
}

void SetCameraInterfacePolarity( unsigned int InvPolPCLK, unsigned int InvPolVSYNC, unsigned int InvPolHREF)
{
	rCIGCTRL	=	(rCIGCTRL & ~((1<<26)|(1<<25)|(1<<24))) | (InvPolPCLK<<26)|(InvPolVSYNC<<25)|(InvPolHREF<<24);
}

void SetCameraWindowOffset( unsigned int WinOfsEn, unsigned int WinHorOfst, unsigned int WinVerOfst, unsigned int WinHorOfst2, unsigned int WinVerOfst2)
{
	rCIWDOFST	=	(rCIWDOFST & ~((0x1<<31)|(0x7ff<<16)|(0x7ff<<0))) | (WinOfsEn<<31)|(WinHorOfst<<16)|(WinVerOfst<<0);

	rCIDOWSFT2	=	(rCIDOWSFT2 & ~((0x7ff<<16)|(0x7ff<<0))) | (WinHorOfst2<<16)|(WinVerOfst2<<0);
}

void SetCameraTestPattern( unsigned int TestPattern)
{
	rCIGCTRL	=	(rCIGCTRL & ~(0x3<<27)) | (TestPattern<<27);
}

void SetImageEffector( unsigned int IE_FIN, unsigned int PAT_Cb, unsigned int PAT_Cr)
{
	rCIIMGEFF	=	(IE_FIN<<26)|(PAT_Cb<<13)|(PAT_Cr<<0);
}

void EnableOverflowIRQ( void)
{
	rCIWDOFST	|=	(1<<30)|(1<<15)|(1<<14)|(1<<13)|(1<<12);
	rCIWDOFST	&=	~((1<<30)|(1<<15)|(1<<14)|(1<<13)|(1<<12));

	rCIGCTRL	|=	(1<<22);
}

void DisableOverflowIRQ( void)
{
	rCIGCTRL	&=	~(1<<22);
}

static void CalculateBurstSize( unsigned int TargetImageSize, unsigned int *MainBurstLength, unsigned int *RemainedBurstLength)
{
	switch((TargetImageSize>>2)&0xf) {
	case 0	:	*MainBurstLength		=	16;
				*RemainedBurstLength	=	16;
				break;
	case 2	: 	*MainBurstLength		=	16;
				*RemainedBurstLength	=	2;
				break;
	case 4	:	*MainBurstLength		=	16;
				*RemainedBurstLength	=	4;
				break;
	case 6	:	*MainBurstLength		=	4;
				*RemainedBurstLength	=	2;
				break;
	case 8	:	*MainBurstLength		=	16;
				*RemainedBurstLength	=	8;
				break;
	case 10	:	*MainBurstLength		=	8;
				*RemainedBurstLength	=	2;
				break;
	case 12	:	*MainBurstLength		=	8;
				*RemainedBurstLength	=	4;
				break;
	case 14	:	*MainBurstLength		=	4;
				*RemainedBurstLength	=	2;
				break;
	default :	Assert(0,"Improper image size\n");
				break;
	}
}

static void CalculatePrescalerRatioShift( unsigned int SRC_Size, unsigned int DST_Size, unsigned int *Ratio,unsigned int *Shift)
{
	if (SRC_Size>=64*DST_Size) 		{	Assert(0,"Out of scale range.\n");	}
	else if (SRC_Size>=32*DST_Size)	{	*Ratio=32;	*Shift=5;	}
	else if (SRC_Size>=16*DST_Size)	{	*Ratio=16;	*Shift=4;	}
	else if (SRC_Size>=8*DST_Size)	{	*Ratio=8;	*Shift=3;	}
	else if (SRC_Size>=4*DST_Size) 	{	*Ratio=4;	*Shift=2;	}
	else if (SRC_Size>=2*DST_Size) 	{	*Ratio=2;	*Shift=1;	}
	else 							{	*Ratio=1;	*Shift=0;	}
}

//*******************************************************************
//*		Functions for Camera Codec Path
//*******************************************************************

void EnableImageCaptureCodec( void)
{
	rCIIMGCPT |= (1<<30);
}

void DisableImageCaptureCodec( void)
{
	rCIIMGCPT &= ~(1<<31);
}

void EnableCodecDMA( void)
{
	rCIIMGCPT |= (1<<24);
}

void DisableCodecDMA( void)
{
	rCIIMGCPT &= ~(1<<24);
}

static unsigned int GetCodecBytesPerPixel( unsigned int OutFormat, unsigned int Interleave)
{
	unsigned int BytesPerPixel;
	
	switch(OutFormat)
	{
	case OutFormat_YCBCR420 :
		BytesPerPixel = 1;
		break;

	case OutFormat_YCBCR422 :
		if (Interleave==INTERLEAVE_OFF)
			BytesPerPixel = 1;
		else
			BytesPerPixel = 2;
		break;

	case OutFormat_RGB16B :
		BytesPerPixel = 2;
		break;

	case OutFormat_RGB24B :
		BytesPerPixel = 4;
		break;
	}
	
	return BytesPerPixel;
}

void SetCodecFormat( unsigned int InFormat, unsigned int OutFormat, unsigned int Interleave, unsigned int Order422)
{
	rCICOTRGFMT	&=	~((0x1<<31)|(0x1<<30)|(0x1<<29));
	rCIIMGCPT	&=	~((0x1<<26)|(0x1<<25));

	switch(OutFormat)
	{
	case OutFormat_RGB16B :
		Assert(!(Interleave==INTERLEAVE_OFF),"Interleave mode must be set in RGB mode.\n");
		rCICOTRGFMT	|=	(InFormat<<31)|(0x1<<30)|(0x1<<29);
		rCIIMGCPT	|=	(0x1<<26)|(0x0<<25);
		break;

	case OutFormat_RGB24B :
		Assert(!(Interleave==INTERLEAVE_OFF),"Interleave mode must be set in RGB mode.\n");
		rCICOTRGFMT	|=	(InFormat<<31)|(0x1<<30)|(0x1<<29);
		rCIIMGCPT	|=	(0x1<<26)|(0x1<<25);
		break;

	case OutFormat_YCBCR420 :
		Assert(!(Interleave==INTERLEAVE_ON),"Could not set Interleave mode in YCBCR420 output format.\n");
		rCICOTRGFMT	|=	(InFormat<<31)|(0x0<<30)|(0x0<<29);
		rCIIMGCPT	|=	(0x0<<26)|(0x0<<25);
		break;

	case OutFormat_YCBCR422 :
		Assert(!(InFormat==InFormat_YCBCR420),"Could not set YCBCR422 output format when the input format is set as YCBCR420.\n");
		rCICOTRGFMT	|=	(InFormat<<31)|(0x1<<30)|(Interleave<<29);
		rCIIMGCPT	|=	(0x0<<26)|(0x0<<25);
		break;
	}

	rCICOCTRL	=	(rCICOCTRL & ~(0x3)) | (Order422);
}

void SetCodecFrameBuffer( unsigned int FrameBuffer, unsigned int TargetHsize, unsigned int TargetVsize, unsigned int OutFormat, unsigned int Interleave)
{
	unsigned int BytesPerPixel;

	if (Interleave==INTERLEAVE_OFF)
	{
		if (OutFormat==OutFormat_YCBCR420)
		{
			rCICOYSA1	=	FrameBuffer;
			rCICOYSA2	=	rCICOYSA1+TargetHsize*TargetVsize+(TargetHsize*TargetVsize/2);
			rCICOYSA3	=	rCICOYSA2+TargetHsize*TargetVsize+(TargetHsize*TargetVsize/2);
			rCICOYSA4	=	rCICOYSA3+TargetHsize*TargetVsize+(TargetHsize*TargetVsize/2);

			rCICOCBSA1	=	rCICOYSA1+TargetHsize*TargetVsize;
			rCICOCBSA2	=	rCICOYSA2+TargetHsize*TargetVsize;
			rCICOCBSA3	=	rCICOYSA3+TargetHsize*TargetVsize;
			rCICOCBSA4	=	rCICOYSA4+TargetHsize*TargetVsize;

			rCICOCRSA1	=	rCICOCBSA1+(TargetHsize*TargetVsize/4);
			rCICOCRSA2	=	rCICOCBSA2+(TargetHsize*TargetVsize/4);
			rCICOCRSA3	=	rCICOCBSA3+(TargetHsize*TargetVsize/4);
			rCICOCRSA4	=	rCICOCBSA4+(TargetHsize*TargetVsize/4);
		} else {	//	OutFormat = YCBCR422
			rCICOYSA1	=	FrameBuffer;
			rCICOYSA2	=	rCICOYSA1+TargetHsize*TargetVsize*2;
			rCICOYSA3	=	rCICOYSA2+TargetHsize*TargetVsize*2;
			rCICOYSA4	=	rCICOYSA3+TargetHsize*TargetVsize*2;

			rCICOCBSA1	=	rCICOYSA1+TargetHsize*TargetVsize;
			rCICOCBSA2	=	rCICOYSA2+TargetHsize*TargetVsize;
			rCICOCBSA3	=	rCICOYSA3+TargetHsize*TargetVsize;
			rCICOCBSA4	=	rCICOYSA4+TargetHsize*TargetVsize;

			rCICOCRSA1	=	rCICOCBSA1+(TargetHsize*TargetVsize/2);
			rCICOCRSA2	=	rCICOCBSA2+(TargetHsize*TargetVsize/2);
			rCICOCRSA3	=	rCICOCBSA3+(TargetHsize*TargetVsize/2);
			rCICOCRSA4	=	rCICOCBSA4+(TargetHsize*TargetVsize/2);
		}
	} else {
		BytesPerPixel = GetCodecBytesPerPixel(OutFormat,Interleave);

		rCICOYSA1	=	FrameBuffer;
		rCICOYSA2	=	rCICOYSA1+(TargetHsize*TargetVsize*BytesPerPixel);
		rCICOYSA3	=	rCICOYSA2+(TargetHsize*TargetVsize*BytesPerPixel);
		rCICOYSA4	=	rCICOYSA3+(TargetHsize*TargetVsize*BytesPerPixel);
	}
}


void SetCodecScalerBypass( void)
{
	rCICOSCCTRL	|=	(1u<<31);
}

void EnableCodecScaler( void)
{
	rCICOSCCTRL |= (1<<15);
}

void DisableCodecScaler( void)
{
	rCICOSCCTRL &= ~(1<<15);
}

void SetCodecScaler( unsigned int SRC_Height, unsigned int SRC_Width, unsigned int DST_Height, unsigned int DST_Width)
{
	unsigned int PreHorRatio,PreVerRatio;
	unsigned int MainHorRatio, MainVerRatio;
	unsigned int H_Shift, V_Shift;

	CalculatePrescalerRatioShift( SRC_Height, DST_Height, &PreHorRatio, &H_Shift);
	CalculatePrescalerRatioShift( SRC_Width, DST_Width, &PreVerRatio, &V_Shift);

	MainHorRatio	=	(SRC_Height<<8)/(DST_Height<<H_Shift);
	MainVerRatio	=	(SRC_Width<<8)/(DST_Width<<V_Shift);

	//	Codec Pre-scaler control
	rCICOSCPRERATIO	=	((10-(H_Shift+V_Shift))<<28)|(PreHorRatio<<16)|(PreVerRatio);
	rCICOSCPREDST	=	((SRC_Height/PreHorRatio)<<16)|(SRC_Width/PreVerRatio);

	//	Codec Main-scaler control
	rCICOSCCTRL		&=	~((1<<31)|(0x1ff<<16)|(0x1ff));
	rCICOSCCTRL		|=	(0<<31)|(MainHorRatio<<16)|(MainVerRatio);

	if (SRC_Height<=DST_Height)	rCICOSCCTRL	|=	(1<<30);
	if (SRC_Width<=DST_Width)	rCICOSCCTRL	|=	(1<<29);
}

void SetCodecDMA( unsigned int TargetHsize, unsigned int TargetVsize, unsigned int OutFormat, unsigned int Interleave)
{
	unsigned int MainBurstLengthY, RemainedBurstLengthY;
	unsigned int MainBurstLengthC, RemainedBurstLengthC;
	unsigned int BytesPerPixel;

	BytesPerPixel = GetCodecBytesPerPixel( OutFormat, Interleave);

	CalculateBurstSize( TargetHsize*BytesPerPixel, &MainBurstLengthY, &RemainedBurstLengthY);
	
	MainBurstLengthC = MainBurstLengthY/2;
	RemainedBurstLengthC = RemainedBurstLengthY/2;

	if (Interleave==1)
	{
		MainBurstLengthY /= 2;
		RemainedBurstLengthY /= 2;
		MainBurstLengthC /= 2;
		RemainedBurstLengthC /= 2;
	}

	//	Codec DMA control
	rCICOCTRL	=	(rCICOCTRL & ~((0x1f<<19)|(0x1f<<14)|(0x1f<<9)|(0x1f<<4))) 
					| (MainBurstLengthY<<19)|(RemainedBurstLengthY<<14)|(MainBurstLengthC<<9)|(RemainedBurstLengthC<<4);

	//	Codec DMA target area.
	rCICOTAREA	=	TargetHsize*TargetVsize;
	
	rCICOTRGFMT	=	(rCICOTRGFMT & ~((0x1fff<<16)|(0x1fff<<0))) | (TargetHsize<<16)|(TargetVsize<<0);

}

void EnableCodecLastIRQ( void)
{
	rCICOCTRL |= (1<<2);
}

void SetCodecFlipMode( unsigned int FlipMd)
{
	rCICOTRGFMT	= (rCICOTRGFMT	& ~(0x3<<14)) | (FlipMd<<14);
}

unsigned int GetCodecFrameCount( void)
{
	return ((rCICOSTATUS>>26)&0x3);	
}

unsigned int GetCodecFrameBuffer( unsigned int FrameNumber)
{
	unsigned int FrameBuffer;
	
	switch (FrameNumber) {
	case 0	:	FrameBuffer = rCICOYSA1;
				break;
	case 1	:	FrameBuffer = rCICOYSA2;
				break;
	case 2	: 	FrameBuffer = rCICOYSA3;
				break;
	case 3	:	FrameBuffer = rCICOYSA4;
				break;
	}

	return (FrameBuffer);
}

//*******************************************************************
//*		Functions for Camera Preview Path
//*******************************************************************

void EnableImageCapturePreview( void)
{
	rCIIMGCPT |= (1<<29);
}

void DisableImageCapturePreview( void)
{
	rCIIMGCPT &= ~(1<<29);
}

static unsigned int GetPreviewBytesPerPixel( unsigned int OutFormat)
{
	unsigned int BytesPerPixel;
	
	switch(OutFormat)
	{
	case OutFormat_RGB16B :
		BytesPerPixel = 2;
		break;

	case OutFormat_RGB24B :
		BytesPerPixel = 4;
		break;
	}

	return BytesPerPixel;
}

void SetPreviewSource( unsigned int SEL_DMA_CAM)
{
	rCIMSCTRL = (rCIMSCTRL & ~(1<<2)) | (SEL_DMA_CAM<<2);
}

void SetPreviewFrameBuffer( unsigned int FrameBuffer, unsigned int TargetHsize, unsigned int TargetVsize, unsigned int OutFormat)
{
	unsigned int BytesPerPixel;

	BytesPerPixel = GetPreviewBytesPerPixel(OutFormat);

	//	PingPong Frame Buffer
	rCIPRCLRSA1	=	FrameBuffer;
	rCIPRCLRSA2	=	rCIPRCLRSA1+TargetHsize*TargetVsize*BytesPerPixel;
	rCIPRCLRSA3	=	rCIPRCLRSA2+TargetHsize*TargetVsize*BytesPerPixel;
	rCIPRCLRSA4	=	rCIPRCLRSA3+TargetHsize*TargetVsize*BytesPerPixel;
}

void EnablePreviewScaler( void)
{
	rCIPRSCCTRL |= (1<<15);
}

void DisablePreviewScaler( void)
{
	rCIPRSCCTRL &= ~(1<<15);
}

void SetPreviewPath( unsigned int SRC_Height, unsigned int SRC_Width, unsigned int TargetHsize, unsigned int TargetVsize, unsigned int OutFormat)
{
	unsigned int PreHorRatio, PreVerRatio;
	unsigned int MainHorRatio, MainVerRatio;
	unsigned int H_Shift, V_Shift;
	unsigned int MainBurstLength, RemainedBurstLength;
	unsigned int BytesPerPixel;


	rCIPRSCCTRL = (rCIPRSCCTRL & ~(1<<30)) | (OutFormat<<30);


	CalculatePrescalerRatioShift( SRC_Height, TargetHsize, &PreHorRatio, &H_Shift);
	CalculatePrescalerRatioShift( SRC_Width, TargetVsize, &PreVerRatio, &V_Shift);
	
	MainHorRatio	=	(SRC_Height<<8)/(TargetHsize<<H_Shift);
	MainVerRatio	=	(SRC_Width<<8)/(TargetVsize<<V_Shift);
	
	//	Preview Pre-scaler control
	rCIPRSCPRERATIO	=	((10-H_Shift-V_Shift)<<28)|(PreHorRatio<<16)|(PreVerRatio);		 
	rCIPRSCPREDST	=	((SRC_Height/PreHorRatio)<<16)|(SRC_Width/PreVerRatio);

	//	Preview Main-scaler control
	rCIPRSCCTRL		&=	~((1<<29)|(1<<28)|(0x1ff<<16)|(0x1ff));
	rCIPRSCCTRL		|=	(MainHorRatio<<16)|(MainVerRatio);

	if (SRC_Height<=TargetHsize)	rCIPRSCCTRL	|=	(1<<29);
	if (SRC_Width<=TargetVsize)		rCIPRSCCTRL	|=	(1<<28);


	BytesPerPixel = GetPreviewBytesPerPixel(OutFormat);

	CalculateBurstSize( TargetHsize * BytesPerPixel, &MainBurstLength, &RemainedBurstLength);

	//	Preview DMA control
   	rCIPRCTRL	=	(rCIPRCTRL	& ~((0x1f<<19)|(0x1f<<14))) | (MainBurstLength<<19)|(RemainedBurstLength<<14);

	//	Preview DMA target area.
	rCIPRTAREA	=	TargetHsize*TargetVsize;
	
	rCIPRTRGFMT	=	(rCIPRTRGFMT & ~((0x1fff<<16)|(0x1fff<<0))) | (TargetHsize<<16)|(TargetVsize<<0);
}

void EnablePreviewLastIRQ( void)
{
	rCIPRCTRL	|=	(1<<2);
}

void SetPreviewCSCRange( unsigned int CSCRange)
{
	rCIPRTRGFMT	=	(rCIPRTRGFMT & ~(0x3<<30)) | (CSCRange<<30);
}

void SetPreviewFlipMode( unsigned int FlipMode)
{
	rCIPRTRGFMT	=	(rCIPRTRGFMT & ~(0x3<<14)) | (FlipMode<<14);
}

unsigned int GetPreviewFrameCount( void)
{
	return (rCIPRSTATUS>>26)&0x3;
}

unsigned int GetPreviewFrameBuffer( unsigned int FrameNumber)
{
	unsigned int FrameBuffer;
	
	switch (FrameNumber) {
	case 0	:	FrameBuffer = rCIPRCLRSA1;
				break;
	case 1	:	FrameBuffer = rCIPRCLRSA2;
				break;
	case 2	: 	FrameBuffer = rCIPRCLRSA3;
				break;
	case 3	:	FrameBuffer = rCIPRCLRSA4;
				break;
	}

	return FrameBuffer;
}

//*******************************************************************
//*		Functions for MSDMA
//*******************************************************************

void SetMSDMAFormat( unsigned int InFormat, unsigned int Interleave, unsigned int Order422)
{
	rCIMSCTRL	=	(rCIMSCTRL & ~((0x1<<5)|(0x3<<3)|(0x1<<1))) | (Interleave<<5)|(Order422<<3)|(InFormat<<1);
}

void SetMSDMAFrameBuffer( unsigned int FrameBuffer, unsigned int SRC_Width, unsigned int SRC_Height, unsigned int SRC_Offset, unsigned int InFormat)
{
	switch(InFormat)
	{
	case MSDMA_InFormat_YCBCR422 :
		rCIMSYSA	=	FrameBuffer+SRC_Offset*2;
		rCIMSYEND	=	rCIMSYSA+(SRC_Width*SRC_Height)*2+SRC_Offset*2*(SRC_Height-1);
		rCIMSYOFF	=	SRC_Offset*2;
		break;

	case MSDMA_InFormat_YCBCR420 :
		rCIMSYSA	=	FrameBuffer+SRC_Offset;
		rCIMSYEND	=	rCIMSYSA+(SRC_Width*SRC_Height)+SRC_Offset*(SRC_Height-1);
		rCIMSYOFF	=	SRC_Offset;

		rCIMSCBSA	=	rCIMSYEND+SRC_Offset;
		rCIMSCBEND	=	rCIMSCBSA+(SRC_Width/2*SRC_Height/2)+SRC_Offset*(SRC_Height/2-1);
		rCIMSCBOFF	=	SRC_Offset;

		rCIMSCRSA	=	rCIMSCBEND+SRC_Offset;
		rCIMSCREND	=	rCIMSCRSA+(SRC_Width/2*SRC_Height/2)+SRC_Offset*(SRC_Height/2-1);
		rCIMSCROFF	=	SRC_Offset;
		break;
	}

	rCIMSWIDTH	=	SRC_Width;
}

void StartMSDMA( void)
{
	//	clear ENVID_MS
	rCIMSCTRL &= ~(1<<0);
	//	Set ENVID_MS
	rCIMSCTRL |= (1<<0);
}

void WaitMSDMA( void)
{
	//	Waiting for EOF_MS
	while (rCIMSCTRL&(1<<6)==0);
}
