/*======================================================================

 Project Name : S3C2443

 Copyright 2006 by Samsung Electronics, Inc.
 All rights reserved.

 Project Description :
 This software is only for verifying functions of the S3C2443. 
 Anybody can use this code without our permission.

 File Name    : PLL_test.c
 Description  : S3C2443 AC97 module code
 Author       : C.S. Jeon
 Dept         : AP
 Created Date : 2006.06.24
 Version      : 0.0
 History
   R0.0 (2006.06.15): C.S.Jeon draft
		- This code is derived from S3C2442 PLL change test code.
		- modified by junon 060623
		
=======================================================================*/

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "def.h"
#include "option.h"
#include "system.h"
#include "console.h"

#include "2443addr.h"
#include "pll.h"
#include "pll_test.h"
//#include "memtest.h"

void Auto_Change_Pll(void);
void Test_PllOnOff(void);
void Test_Locktime(void);
void Test_EPLL_Locktime(void);
void Test_Clock_Divide(void);
void Change_Voltage(void);

// Sub function
U32 Calc_Locktime(U32 lock_time);


#if FIN==12000000
volatile unsigned int Mps_Val[][4]={
// MPLL_CLK,	 M,  P,  S.
	{300000000, 17,  1, 1},
	{300000000, 17,  1, 1},
	{348000000, 21,  1, 1},
	{348000000, 21,  1, 1},

	{402000000, 59,  2, 1},
	{402000000, 59,  2, 1},
	{420000000, 27,  1, 1},
	{420000000, 27,  1, 1},
	{432000000, 28,  1, 1},
	{438000000, 65,  2, 1},
	{450000000, 67,  2, 1},
	{462000000, 69,  2, 1},
	{468000000, 31,  1, 1},
	{480000000, 32,  1, 1},
	{492000000, 33,  1, 1},
	{498000000, 75,  2, 1},

	{510000000, 77,  2, 1},
	{510000000, 77,  2, 1},
	
	{522000000, 79,  2, 1},
	{522000000, 79,  2, 1},
	{528000000, 36,  1, 1},
	{534000000, 81,  2, 1},
	{534000000, 81,  2, 1},
	{540000000, 37,  1, 1},
	{546000000, 83,  2, 1},
	{560000000, 62,  3, 0},
	{568000000, 63,  3, 0},
	{576000000, 64,  3, 0},
	{584000000, 65,  3, 0},
	{592000000, 66,  3, 0},

	{600000000, 17,  1, 0},
	{612000000, 43,  2, 0},
	{624000000, 18,  1, 0},
	{632000000, 71,  3, 0},
	{640000000, 72,  3, 0},
	{648000000, 19,  1, 0},
	{660000000, 47,  2, 0},
	{672000000, 20,  1, 0},	
	{688000000, 78,  3, 0},

	{708000000, 51,  2, 0},
	{720000000, 22,  1, 0},
	{752000000, 86,  3, 0},
	{780000000, 57,  2, 0},

	{800000000, 92,  3, 0},
	{852000000, 63,  2, 0},
	{900000000, 67,  2, 0},

	{948000000, 71,  2, 0},

	{1008000000, 34,  1, 0},
	{1056000000, 36,  1, 0},
//	{1092000000, 83,  2, 0},

};

#endif

int No_Of_Mps_Val=sizeof(Mps_Val)/16;


void * func_pll_test[][2]=
{   
	//  						"123456789012345678901"
   	(void *)Test_Locktime,		"MPLL Locktime Test   ",
   	(void *)Test_EPLL_Locktime,	"EPLL Locktime Test   ", 
   	(void *)Test_Clock_Divide,	"Clock Divider Test   ", 
   	(void *)Manual_Change_Pll,	"Manual PLL Test      ", 
   	(void *)Auto_Change_Pll,	"Auto PLL Test        ", 
   	(void *)Test_PllOnOff,		"PLL On/Off Test      ", 
   	(void *)Change_Voltage,		"Arm voltage test     ",
   	(void *)System_Burden_Test,	"burden test       ", 
   	0,0
};


void Test_PLL(void)
{
	int i;
	
	
	rGPHCON=rGPHCON&~(0xf<<26)|(0xa<<26);
	//                                   CLKOUT1   CLKOUT0
	rMISCCR=rMISCCR&~((0x7<<8)|(0x7<<4))|(0x0<<8)|(0x1<<4);
	rCLKDIV0|=(1<<14);
		
	while(1)
	{
		i=0;
		printf("\n\n=============== PLL Function Test ===============\n\n");
		
		while(1)
		{   //display menu
			printf("%2d:%s",i,func_pll_test[i][1]);
			i++;
			if((int)(func_pll_test[i][0])==0)
			{
				printf("\n");
				break;
			}
			if((i%4)==0)
			printf("\n");
		}
		printf("\n======================================================\n");
		printf("Select #Item or Press Enter key to exit : ");
		i = GetIntNum(); 
		if(i==-1) break;		// return.
		if(i>=0 && (i<((sizeof(func_pll_test)-1)/8)) )	// select and execute...
			( (void (*)(void)) (func_pll_test[i][0]) )();
	}
	
}


// lock time: user lock time setting time[us],
// lock_count: register setting value for wanted locktime.
#define TIME_OF_XTAL	((float)1000000/(float)FIN)	//us

U32 Calc_Locktime(U32 lock_time)
{
	U32 lock_count=0;

	lock_count = (U32)((float)lock_time/((float)TIME_OF_XTAL));

	return lock_count;
}

void Test_Locktime(void)
{
	int temp;
	int i,j, mval, pval, sval, fclk, hdiv_val, pdiv_val;
	U32 lock_time=300, lock_count=12048;	//Unit is us.
	U32 mpllcon_bk;

	// clock register backup.
	mpllcon_bk = rMPLLCON;

	printf("[MPLL Lock time test 100,000 times]\n");
	printf("Lock time range: 300us - %d [us]\n", (U32)(0xffff*TIME_OF_XTAL));
	printf("Input Lock time[us]: ");
	lock_time = GetIntNum();

	lock_count = Calc_Locktime(lock_time);	// lock_time [us]
	rLOCKCON0=(lock_count<<0); 
	printf("Locktime register value is %d(%xh).\n", rLOCKCON0&0xffff);
	
//	printf("Clock will be locked after LED7 is ON.\n");

	i=0;
		
	while(1) 
	{
		if(Uart_GetKey()!=0x0) 
			break;
		if(i>100000) 
			break;
		if(i%2) 
		{
			Led_Display(0x8);
			SetMPLL( 92, 3, 1);
			SystemCLK(0);
			ChangeSDRAMParameter(HCLK);
			Console();
			printf("[400MHz]\n");
		} 
		else 
		{		
			Led_Display(0x0);
			SetMPLL( 17, 1, 1);
			SystemCLK(0);
			ChangeSDRAMParameter(HCLK);
			Console();
			printf("[300MHz]\n");
		}
		//Delay(1000);
		i++;
	}	
	printf("[PLL Lock time test %d times end]\n",i-1);
}

void Test_EPLL_Locktime(void)
{
	int temp;
	int i,j, mval, pval, sval, fclk, hdiv_val, pdiv_val;
	U32 lock_time=150, lock_count=12048;	//Unit is us.
	U32 epllcon_bk;

	// clock register backup.
	epllcon_bk = rEPLLCON;
    
    
    printf("EPLLCON: %x.\n", rEPLLCON);
	printf("[EPLL Lock time test 100,000 times]\n");
	printf("Lock time range: 150us - %d [us]\n", (U32)(0xffff*TIME_OF_XTAL));
	printf("Input Lock time[us]: ");
	lock_time = GetIntNum();
	
	


	lock_count = Calc_Locktime(lock_time);	// lock_time [us]
	rLOCKCON1=(lock_count<<0);
	rCLKSRC|=(1<<6);  // EPLL Output
	SetEPLL( 40, 1, 1);
	rEPLLCON&=~(1<<24);  // EPLL ON 
	

	printf("Locktime register value is %d(%xh).\n", rLOCKCON1&0xffff, rLOCKCON1&0xffff);
    printf("CLKSRC:%x EPLLCON: %x", rCLKSRC, rEPLLCON);
    
    getchar();
	
//	printf("Clock will be locked after LED7 is ON.\n");

	i=0;
		
	while(1) 
	{
		if(Uart_GetKey()!=0x0) 
			break;
		if(i>100000) 
			break;
		if(i%2) 
		{
			Led_Display(0x8);
			SetEPLL( 40, 1, 1); // EPLL 96MHz
			printf(".");
	  	} 
		else 
		{		
			Led_Display(0x2);
			SetEPLL( 40, 1, 2);	 // EPLL 96MHz
			printf(",");
		}
		//Delay(1000);
		i++;
	}	
	printf("[PLL Lock time test %d times end]\n",i-1);
}

void Test_Clock_Divide(void)
{
	int temp;
	int i,j, mval, pval, sval, fclk, hdiv_val, pdiv_val;
	U32 clkdiv0_bk;


    clkdiv0_bk=rCLKDIV0;
	   
	
	Uart_TxEmpty();
	SetCLKDIV( 0, 3, 3, 1);		 // 1:16:16
	SystemCLK(0);
	ChangeSDRAMParameter(HCLK);
	Console();
	
	
	printf("\n[Clock Divide Test]\n");
	printf("Probe CLKOUT0 and check FCLK, HCLK, PCLK\n");

	rGPHCON=rGPHCON&~(3<<26)|(2<<26);    // CLKOUT0 setting
	rGPHUDP&=~(3<<26);  // pull updown disable
	rMISCCR=rMISCCR&~(7<<4)|(2<<4);     // 2: FCLK, 3:HCLK, 4:PCLK
	printf("MPLLOUT is 400MHz\n");	
	printf("ARMCLK=MPLLOUT \n ");
	
	rCLKDIV0=rCLKDIV0&~(0xf<<9)|(0<<9);
	getchar();
	SystemCLK(0);
	Console();

	printf("ARMCLK=MPLLOUT /2 \n ");
	rCLKDIV0=rCLKDIV0&~(0xf<<9)|(0x8<<9);
	getchar();
	SystemCLK(0);
	Console();

	printf("ARMCLK=MPLLOUT /3 \n ");
	rCLKDIV0=rCLKDIV0&~(0xf<<9)|(0x2<<9);
	getchar();
	SystemCLK(0);
	Console();

	printf("ARMCLK=MPLLOUT /4 \n ");
	rCLKDIV0=rCLKDIV0&~(0xf<<9)|(0x9<<9);
	getchar();
	SystemCLK(0);
	Console();
 
	printf("ARMCLK=MPLLOUT /6 \n ");
	rCLKDIV0=rCLKDIV0&~(0xf<<9)|(0xa<<9);
	getchar();
	SystemCLK(0);
	Console();

	printf("ARMCLK=MPLLOUT /8 \n ");
	rCLKDIV0=rCLKDIV0&~(0xf<<9)|(0xb<<9);
	getchar();
	SystemCLK(0);
	Console();

	printf("ARMCLK=MPLLOUT /12 \n ");
	rCLKDIV0=rCLKDIV0&~(0xf<<9)|(0xd<<9);
	getchar();
	SystemCLK(0);
	Console();

	printf("ARMCLK=MPLLOUT /16 \n ");
	rCLKDIV0=rCLKDIV0&~(0xf<<9)|(0xf<<9);
	getchar();
	SystemCLK(0);
	Console();

    printf("MPLL:ARMCLK:HCLK:PCLK = 400MHz:400MHz:100MHz:50MHz\n ");
    printf("CLKOUT0 => HCLK \n ");
    rMISCCR=rMISCCR&~(7<<4)|(3<<4);     // 2: FCLK, 3:HCLK, 4:PCLK
    printf("HCLK=MPLLOUT/4 (400MHz /4) \n ");
    
    rCLKDIV0=rCLKDIV0&~(0xf<<9)|(0x0<<9);
	rCLKDIV0=rCLKDIV0&~((0x3<<4)|(0x3<<0))|(0x1<<4)|(1<<0);
	SystemCLK(0);
	ChangeSDRAMParameter(HCLK);
	Console();
	getchar();

	printf("HCLK=MPLLOUT /6 (400MHz /6) \n ");
	
	rCLKDIV0=rCLKDIV0&~((0x3<<4)|(0x3<<0))|(0x2<<4)|(1<<0);
	getchar();
	SystemCLK(0);
	ChangeSDRAMParameter(HCLK);
	Console();

	printf("HCLK=MPLLOUT /8(400MHz /8) \n ");
	
	rCLKDIV0=rCLKDIV0&~((0x3<<4)|(0x3<<0))|(0x3<<4)|(1<<0);  
	getchar();
	SystemCLK(0);
	ChangeSDRAMParameter(HCLK);
	Console();

	printf("HCLK=MPLLOUT /12(400MHz /12) \n ");

	rCLKDIV0=rCLKDIV0&~((0x3<<4)|(0x3<<0))|(0x2<<4)|(3<<0);  
	getchar();
	SystemCLK(0);
	ChangeSDRAMParameter(HCLK);
	Console();

	printf("HCLK=MPLLOUT /16(400MHz /16) \n ");
	
	rCLKDIV0=rCLKDIV0&~((0x3<<4)|(0x3<<0))|(0x3<<4)|(3<<0); 
	getchar();
	SystemCLK(0);
	ChangeSDRAMParameter(HCLK);
	Console();
	
	
	printf("CLKOUT0 => PCLK \n ");
	
	rMISCCR=rMISCCR&~(7<<4)|(4<<4); 	// 2: FCLK, 3:HCLK, 4:PCLK
		
	printf("PCLK=HCLK (400MHz /16) \n ");
	
	rCLKDIV0=rCLKDIV0&~(0x1<<2)|(0x0<<2);  
	getchar();
	SystemCLK(0);
	Console();

	printf("PCLK=HCLK /2 (400MHz /32) \n ");
	
	rCLKDIV0=rCLKDIV0&~(0x1<<2)|(0x1<<2);  
	getchar();
	
	

    rCLKDIV0=rCLKDIV0&~(0xf<<9)|(0x0<<9);
	rCLKDIV0=rCLKDIV0&~((0x3<<4)|(0x3<<0))|(0x1<<4)|(1<<0);
	
    SystemCLK(0);
	ChangeSDRAMParameter(HCLK);
	Console();
	
	printf("[Clock divide test end]\n");
}


void Manual_Change_Pll(void)  // modified by junon 060623
{
	U32 pval, mval, sval, s_temp, fclk, armdiv, prediv, hclkdiv, pclkdiv, voltage=120;
	unsigned int i, fclk_sel;
	float ffclk;

	printf("[Running change test of M/P/S value]\n");
	SystemCLK(1); // current clock setting

	// clock out selection
	rGPHCON = (rGPHCON&~(0xf<<26))|(0xa<<26); // use CLKOUT0, CLKOUT1 
	rMISCCR = (rMISCCR&~(0xff<<4))|(3<<8)|(2<<4); // [10:8]=CLKSEL1-3:HCLK, [6:4]=CLKSEL0-2:ARMCLK

	for(i=0; i<sizeof(Mps_Val)/16; i++) {
		if(i%2==0) printf("\n");
		printf("%2d: MPLL=%4.1f [%xh(%d),%xh,%xh]	", i, (float)Mps_Val[i][0]/1000000,Mps_Val[i][1],Mps_Val[i][1],Mps_Val[i][2],Mps_Val[i][3]);
	}
	printf("\n");

	#if 1
	printf("Select MPLL No(0-%d): ", sizeof(Mps_Val)/16-1);
	fclk_sel=GetIntNum();
	mval = Mps_Val[fclk_sel][1];
	pval = Mps_Val[fclk_sel][2];
	sval = Mps_Val[fclk_sel][3];
	#else
	printf("Input M vlaue : ");
	mval=GetIntNum();		
	printf("Input P vlaue : ");
	pval=GetIntNum();	
	printf("Input S vlaue : ");
	sval=GetIntNum();
	#endif

	printf("Select ARM divider [0=1/1, 8=1/2, 2=1/3, 9=1/4, 10=1/6, 11=1/8, 13=1/12, 15=1/16] : ");
	armdiv=GetIntNum();
	printf("Select Pre divider [0=1/1, 1=1/2, 2=1/3, 3=1/4] : ");
	prediv=GetIntNum();
	printf("Select HCLK divider [0=1/1, 1=1/2, 3=1/4] : ");
	hclkdiv=GetIntNum();
	printf("Select PCLK divider [0=1/1, 1=1/2] : ");
	pclkdiv=GetIntNum();

	// calculate MPLL clock
	s_temp=(int)pow(2,sval);
	ffclk = (U32) ( (((float)mval+8)*FIN*2.0)/(((float)pval)*(float)s_temp));	// pll3000x.
	printf("Now change PLL value...(%dhz)[mps=%xh(%d), %d, %d]\n", (U32)ffclk, mval,mval, pval, sval);

/*	printf("ARM core voltage? [ex)1.35V -> 135] : ");
	voltage = GetIntNum();
	if (voltage > 180) 
	{
		printf("ARM core voltage limit is 1.5V, then set 1.5V\n");
		voltage = 150;
	}
	Max1718_Set(1, voltage); // pwr=1:ARM, pwr=0:INT
	
	Delay(10);
*/
	SetCLKDIV(armdiv, prediv, hclkdiv, pclkdiv);
	SetMPLL(mval, pval, sval);

	SystemCLK(1);
	ChangeSDRAMParameter(HCLK);
	Console();
	
	printf("press 'q' key to exit\n");	
	while(Uart_GetKey()!='q')
	{
		Led_Display(9);
		for(i=0;i<100000;i++);
		Led_Display(6);
		for(i=0;i<100000;i++);
	}
								   
	#if 0
	//Uart_Getch();
	printf("Memory test (y/n)? ");
	if(getc()=='y') {
		while(1) {
//			Mem_Test(1);
			if(Uart_getc()!=0x0) break;
			//Uart_SendByte((int)'.');
		}
	}
	#endif
	
}


void Auto_Change_Pll(void)
{
	int temp, voltage=145;
	int i,j, armdiv, prediv, hdiv, pval, sval, fclk, hdiv_val, pdiv_val;
	U32 prev_fclk;
	U32 mpllcon_bk, clkdivn_bk, camdivn_bk;

	// clock register backup.
	mpllcon_bk = rMPLLCON;
	clkdivn_bk = rCLKDIV0;
		
	
	printf("[PLL frequency change test 100,000 times] = %d depth\n",No_Of_Mps_Val);
	Uart_TxEmpty();

	rLOCKCON0=0xe10;    // MPLL Lock Time
	// 3614 is 300us at 12Mhz
	printf("locktime=0x%x, \n",rLOCKCON0);

	j=0;
	while(1) 
	{
		if(Uart_GetKey() == 'q') 
			break;
//		if(j>100000) 
//			break;	

		i= j%No_Of_Mps_Val;


		fclk = Mps_Val[i][0];
//		printf("[%d] ", j);
		Uart_TxEmpty();


		if(prev_fclk==(Mps_Val[i][0]/(armdiv+1)))
		{
			armdiv=0;
			prediv=1;
			hdiv=1;
			pval=1;
		}
		else
		{
			
	    if (Mps_Val[i][0] >= 800000000 && Mps_Val[i][0] < 1100000000) // 1:6:12
		{
			armdiv=8;
			prediv=3;
			hdiv=1;
			pval=1;
			
		}
		else if (Mps_Val[i][0] >= 666000000 && Mps_Val[i][0] < 800000000) // 1:6:12
		{
			armdiv=8;
			prediv=2;
			hdiv=1;
			pval=1;
			
		}
		else if (Mps_Val[i][0] >= 535000000 && Mps_Val[i][0] < 666000000) // 1:6:12
		{
			armdiv=8;
			prediv=2;
			hdiv=1;
			pval=1;
			
		}
		else if (Mps_Val[i][0] >= 420000000 && Mps_Val[i][0] < 535000000) // 1:6:12
		{
			armdiv=0;
			prediv=2;
			hdiv=1;
			pval=1;

		}
		else if (Mps_Val[i][0] >= 266000000 && Mps_Val[i][0] < 420000000) // 1:3:6
		{
			armdiv=0;
			prediv=2;
			hdiv=0;
			pval=1;

		}
		else		
		{
			armdiv=0;
			prediv=2;
			hdiv=1;
			pval=1;

		}
		
		}
		
	
		Led_Display(j%2);
		
		Delay(10);

		if(prev_fclk<(Mps_Val[i][0]/(armdiv+1))) 
		{
			Led_Display(0x1);		

			SetCLKDIV( armdiv, prediv, hdiv, pval);
			Led_Display(0x2);
			SetMPLL(Mps_Val[i][1],Mps_Val[i][2],Mps_Val[i][3]);

		} 
		else 
		{
			Led_Display(0x2);

			SetMPLL(Mps_Val[i][1],Mps_Val[i][2],Mps_Val[i][3]);
			Led_Display(0x1);		
			SetCLKDIV( armdiv, prediv, hdiv, pval);
		}

		
		SystemCLK(0);
		Led_Display(0x4);
		ChangeSDRAMParameter(HCLK);
		Led_Display(0x8);
		Console();
		printf("[%d]ARMCLK=%d,HCLK=%d [Mps_Val=%xh,%xh,%xh]\n", j, ARMCLK,HCLK,Mps_Val[i][1],Mps_Val[i][2],Mps_Val[i][3]);

		prev_fclk = ARMCLK;
		j++;
	}  

	printf("\nMPLL change test OK.\n");

	printf("\nReturn to Previous setting.\n");
	rMPLLCON = mpllcon_bk;
	rCLKDIV0 = clkdivn_bk;
	
	SystemCLK(1);
	Console();

}


void Test_PllOnOff(void)
{
	int i,j;
	unsigned int saveREFRESH, saveCLKSRC, saveCLKDIV0;

	printf("[PLL On/Off test, 10000 times]\n");
	Uart_TxEmpty();	//To avoid being crushed the character
	
	saveCLKSRC = rCLKSRC;
    saveCLKDIV0 = rCLKDIV0;

	rLOCKCON0=0xe10; // 0xe10 is 300us at 12Mhz
	rLOCKCON1=0x710; // 0x710 is 150us at 12Mhz
	
	
	rCLKSRC|=(1<<6);  // EPLL Output
	SetEPLL( 40, 1, 1);
	rEPLLCON&=~(1<<24);  // EPLL ON
	
 

	for(j=0;j<10000;j++) {
		Led_Display(0x2);
		
	
		rCLKSRC=rCLKSRC & ~((1<<4)|(1<<3));
		rCLKDIV0&=~((0xf<<9)|(0x3<<4)|(1<<2)|(0x3<<0)); // FCLK=reference clock,  ARMCLK:HCLK:PCLK=1:1:1
		rMPLLCON|=(1<<24);  // MPLL off
		rEPLLCON|=(1<<24);  // EPLL Off 
	 	// Re-setting the refresh count for 12MHz HCLK.
		saveREFRESH=rREFRESH;
		rREFRESH=(U32)(2048+1-12*7.8); 
	    
	    
	    Delay(100); //
	    rMPLLCON&=~(1<<24);  // MPLL on
	    rEPLLCON&=~(1<<24);  // EPLL ON 
	    
	    Led_Display(0x8);		
		rCLKDIV0 = saveCLKDIV0;
		rCLKSRC = saveCLKSRC;
    	rREFRESH=saveREFRESH;
   		Delay(1000); //
  		
	}
	rCLKDIV0 = saveCLKDIV0;
	rCLKSRC = saveCLKSRC;
   	rREFRESH=saveREFRESH;

	
}


void Change_Voltage(void)
{
    int voltage;

    printf("Select core voltage? [0.8V -> 1.75V] : ");
	printf("ARM core voltage? [ex)1.35V -> 135] : ");
	voltage = GetIntNum();
	if (voltage > 180) 
	{
		printf("ARM core voltage limit is 1.5V, then set 1.5V\n");
		voltage = 150;
	}
	Max1718_Set(1, voltage); // pwr=1:ARM, pwr=0:INT
	
	Delay(10);
	
	
}


int Burden_Test(void)
{
    int i,j,k,l,m,err=0, size, count;

    int *pt_nor1, *pt_nor2;
    
    printf("SDRAM Write Test\n");
    size = 0x400000;//16MB
    count = 0x10000;

    pt_nor1=(int *)0x30200000;//CNB(WT Cache, on buffer)
    pt_nor2=(int *)0x31200000;//NCNB(Off cache, off buffer)

    for(i=0;i<size;i++)
    {
  	*pt_nor1++=(U32)i;
	if((i%(count*10))==0)	
	    Dhry_run();
	*pt_nor2++=(U32)i;
	if((i%(count*10))==0)	
	    Dhry_run();
	if((i%(count*10))==0)	
	    Dhry_run();
	if((i%(count*10))==0)	
	    Dhry_run();
    	if((i%count)==0)    
	    printf(".");
	if(Uart_GetKey())
	    return 0;
    }
    printf("\nWriting is completed.\n");

    pt_nor1=(int *)0x30200000;
    pt_nor2=(int *)0x31200000;

    for(i=0;i<size;i++)
    {
        j=*pt_nor1++;
	if((i%(count*10))==0)	
	    Dhry_run();
	k=*pt_nor2++;
	if((i%(count*10))==0)	
	    Dhry_run();

 	if((i%(count*10))==0)	
	    Dhry_run();

	if((i%(count*10))==0)	
	    Dhry_run();

	if(j!=(U32)i)
	{
	    printf("%x=%x,%x\n",(U32)pt_nor1,j,i);
	    err++;
    	}
		
    	if(k!=(U32)i)
	{
	    printf("%x=%x,%x\n",(U32)pt_nor2,k,i);
	    err++;
    	}

    	if((i%count)==0)printf(".");
	if(Uart_GetKey())
	    return 0;
    }
    if(err==0)
    	printf("\nMemory read test:O.K.\n");
    else
    {
	printf("\nMemory read test:FAIL!!!\n");
	Led_Display(0xf);
	while(1);
    }
    return 1;
}

void __irq Burden_Done(void);

volatile unsigned int var0=0;

void Timer_Burden(void)
{
    unsigned int Cnt;
    var0 = 0;
    
    printf("\n timer setting\n");

    rINTMSK = ~(BIT_TIMER0);
    pISR_TIMER0 = (int)Burden_Done;

    rTCFG0 = 0xff;  //Dead zone=0,Prescaler0=256(0xff)
    rTCFG1 = 0x3;   //All interrupt,Mux0=1/16

    Cnt = PCLK/256/16;
    rTCNTB0 = (25 * Cnt) / 1000;           //(1/(50MHz/16/16)) * 0x3333 (13107) = 0.334s ( 1.124Hz)
    rTCMPB0 = (25 * Cnt) / 2000;

    rTCON = 0xa;         //Auto reload, Inverter off, Manual update, Dead zone disable, Stop  
    rTCON = rTCON & ~(0xffffff) | 0x9;
    
   
    
}

void __irq Burden_Done(void)
{
    rSRCPND = BIT_TIMER0;       //Clear pending bit
    rINTPND = BIT_TIMER0;
    rINTPND;                    //Prevent an double interrupt pending

    Led_Display(var0%16);
//    printf("^");
    var0++;    
}

void System_Burden_Test(void)
{
    int i;
    
    rGPHCON=rGPHCON&~(0xf<<26)|(0xa<<26);
	rMISCCR=rMISCCR&~(0x7<<4)|(0x4<<4);
    Timer_Burden();
    
    while(Burden_Test());
    rTCON   = 0x0;
    rINTMSK |= BIT_TIMER0;
}

